;*******************************************************************************************
;   Beschreibung:	Zhler eHZ MT691 auslesen, 4MHZ Quarz																								 *							 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	Solaranlage_P16F1827_V20.asm						                               		 *
;    Datum:        	21.01.2024                                                            *
;    Version: 		V2.0                                                                  *
;                                                                                          *
;    Autor:       	Werner Hirsch                                                       *
;    Internet:      	www.wernerhirsch.de                                                           *
;    e-mail:		wernerhirschhh@aol.com                                                          *
;                                                                                          *
;*******************************************************************************************
;	Subtraktion: STATUS,C = 0, wenn Ergebniss <0, = 1, wenn Ergebniss 0 oder >0*
;	Addition: STATUS,C = 0, wenn Ergebniss <256, = 1, wenn Ergebniss >255*
;
;
;																				 *
;													 *
;                                                                                          *
;*******************************************************************************************


	list      p=16f1827			;definiert den verwendeten Prozessor
	#include <p16f1827.inc>		;in diesem File stehen die Adressen und Bitnamen

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES
	
   __CONFIG _CONFIG1, _FOSC_XT & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_OFF & _BORV_19 & _LVP_OFF
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus


;===========================
;  DEFINITIONEN
;===========================

#define DISP_EN			PORTA, 0		;(pin 17) Mit der positiven Flanke von DISP_EN werden die parallelen Daten bernommen
#define DISP_RS			PORTA, 1		;(pin 18) Unterscheidung zwischen Befehl und Daten
#define SI_CLK			PORTA, 2		;(pin 1) Serieller Takt fr Display
#define SI_DAT			PORTA, 3		;(pin 2) Serielle Daten zum Display
#define IN_PROD			PORTA, 4		;(pin 3) Eingang Erzeugung (CCP4)
#define IN_NC_RA5		PORTA, 5		;(pin 4) Vpp
#define IN_NC_RA6		PORTA, 6		;(pin 15) OSC
#define IN_NC_RA7		PORTA, 7		;(pin 16) OSC
#define OUT_PWM			PORTB, 0		;(pin 6) Ausgang PWM
#define IN_RX			PORTB, 1		;(pin 7) Eingang Rx (optischer Sensor)
#define OUT_ON_OFF		PORTB, 2		;(pin 8) Ausgang Ein/Aus (Bei Einspeisung Ein, Bei Bezug nach xx min aus: T_Absch), Tx
#define IN_TEMP			PORTB, 3		;(pin 9) Eingang Temperatur AN9
#define IN_MENUE		PORTB, 4		;(pin 10) Eingang Men
#define IN_OK			PORTB, 5		;(pin 11) Eingang OK
#define IN_UP			PORTB, 6		;(pin 12), PGC, Eingang Up
#define IN_RECHTS		PORTB, 7		;(pin 13), PGD, Eingang Rechts
										;PortA: 11110000; PortB: 11111110


;===========================
;  KONSTANTEN
;===========================


;===========================
;  VARIABLEN BANK 0
;===========================

LOOP_1			EQU 0x20		;Zhler fr Wartezeit
LOOP_2			EQU 0x21		;Zhler fr Wartezeit
SHIFT_REG		EQU	0x22		;Datum, welches an das Display gesendet werden soll
SHIFT_CNT		EQU	0x23		;Zhler fr die Datenausgabe
TEMP_11			EQU	0x24		;Allgemeines Zwischenspeicherregister
TEMP_12			EQU	0x25		;Allgemeines Zwischenspeicherregister
TEMP_13			EQU	0x26		;Allgemeines Zwischenspeicherregister
TEMP_14			EQU	0x27		;Allgemeines Zwischenspeicherregister
TEMP_15			EQU	0x28		;Allgemeines Zwischenspeicherregister
DISP_7			EQU 0x29		;Anzeigeregister Digit 8
DISP_6			EQU 0x2A		;Anzeigeregister Digit 7
DISP_5			EQU 0x2B		;Anzeigeregister Digit 6
DISP_4			EQU 0x2C		;Anzeigeregister Digit 5
DISP_3			EQU 0x2D		;Anzeigeregister Digit 4
DISP_2			EQU 0x2E		;Anzeigeregister Digit 3
DISP_1			EQU 0x2F		;Anzeigeregister Digit 2
DISP_0			EQU 0x30		;Anzeigeregister Digit 1
MENU_COUNT		EQU 0x31		;
M_HOR_CNT		EQU 0x32		;Menpunktzhler horizontal
M_HOR_CNT_MAX	EQU 0x33		;Menpunktzhler horizontal max
MENU_STAT		EQU 0x34		;Statusregister:	Bit0: Update Bit Menpunkte (horizontal)
								;					Bit1: 0=Vertik. Menpunkt -, 1=Vertik. Menpunkt +
								;					Bit7: Update Bit Hauptmen (vertikal)
STAT_TIMER		EQU 0x35		;Status Register Timer: Bit 0/1: 8ms, Bit 2: 1s, Bit 3/4: 4s, Bit 5: 1Tag, Bit 6:
STAT_DATEN		EQU 0x36		;Statusregister:	Bit0: neuer Impuls von Prod Zhler
								;					Bit1: berlauf bit fr 32 Bit Timer 1
								;					Bit2: Datenbertragung USART: 1= noch nicht beendet, 0= noch nicht gestartet
								;					Bit3: berlaufbit Speicherbereich 
								;					Bit4: Wechsel Displayanzeige: 0= automatisch, 1= manuell
								;					Bit5: Leistung Pb/e: 0= positiv (Bezug), 1= negativ (Einspeisung)
SEKUNDE			EQU 0x37		;Register interne Uhr, Sekunde
MINUTE			EQU 0x38		;Register interne Uhr, Minute
STUNDE			EQU 0x39		;Register interne Uhr, Stunde

JAHR			EQU	0x3A		;Register fr Jahr 20XX
MONAT			EQU	0x3B		;Register fr Monat
TAG				EQU	0x3C		;Register fr Tag
BEFEHL			EQU	0x3D		;Register fr Tasten-/IR-Befehle
EEPROM_ADR		EQU 0x3E		;Speicheradresse EEPROM fr Zhlerstnde
TMR1_EXT1		EQU 0x3F		;Timer 1 Erweiterung
TMR1_EXT2		EQU 0x40		;Timer 1 Erweiterung
TMR2_EXT		EQU	0x41		;Timer 2 Erweiterung
T_PROD_0		EQU 0x42		;Alter Stand Timer 1, Byte 0
T_PROD_1		EQU 0x43		;Alter Stand Timer 1, Byte 1
T_PROD_2		EQU 0x44		;Alter Stand Timer 1, Byte 2
T_PROD_3		EQU 0x45		;Alter Stand Timer 1, Byte 3
ZAEHL_BEZUG_3	EQU 0x46		;Zhlerstand Bezug (1.8.0), Bit 24-31
ZAEHL_BEZUG_2	EQU 0x47		;Zhlerstand Bezug (1.8.0), Bit 16-23
ZAEHL_BEZUG_1	EQU 0x48		;Zhlerstand Bezug (1.8.0), Bit 8-15
ZAEHL_BEZUG_0	EQU 0x49		;Zhlerstand Bezug (1.8.0), Bit 0-7
ZAEHL_EINSP_3	EQU 0x4A		;Zhlerstand Einspeisung (2.8.0), Bit 24-31
ZAEHL_EINSP_2	EQU 0x4B		;Zhlerstand Einspeisung (2.8.0), Bit 16-23
ZAEHL_EINSP_1	EQU 0x4C		;Zhlerstand Einspeisung (2.8.0), Bit 8-15
ZAEHL_EINSP_0	EQU 0x4D		;Zhlerstand Einspeisung (2.8.0), Bit 0-7
ZAEHL_PROD_3	EQU 0x4E		;Zhler Produktion, Bit 24-31
ZAEHL_PROD_2	EQU 0x4F		;Zhler Produktion, Bit 16-23
ZAEHL_PROD_1	EQU 0x50		;Zhler Produktion, Bit 8-15
ZAEHL_PROD_0	EQU 0x51		;Zhler Produktion, Bit 0-7
P_INOUT_H		EQU 0x52		;Leistung Bezug/Einspeisung (16.7.0), H Byte
P_INOUT_M		EQU 0x53		;Leistung Bezug/Einspeisung (16.7.0), M Byte
P_INOUT_L		EQU 0x54		;Leistung Bezug/Einspeisung (16.7.0), L Byte
P_PROD_H		EQU 0x55		;Leistung Produktion, H Byte
P_PROD_L		EQU 0x56		;Leistung Produktion, L Byte
FSR_REG_0		EQU 0x57		;FSR 0 Tempregister fr Interrupt
FSR_REG_1		EQU 0x58		;FSR 1 Tempregister fr Interrupt
FSR_REG_0H		EQU 0x59		;FSR 0H Tempregister fr Interrupt
ZPROD_PRE		EQU 0x5A		;Vorteiler fr Produktionszhler, Zhler
ZPROD_IMP		EQU 0x5B		;Vorteiler fr Produktionszhler, Rcksetzwert, 10-100 => 1.000 Imp/kWh - 10.000 Imp/kWh
REGLER_KP		EQU 0x5C		;Kp Wert fr Regler Heizung, Kp: 0-0,99 (0,5)
REGLER_TN		EQU 0x5D		;Tn Wert fr Regler Heizung, Tn: 1-100 (5)
P_LAST_H		EQU 0x5E		;Max. Leistung Heizung, H-Byte, 100% => 1-32767 (600)
P_LAST_L		EQU 0x5F		;Max. Leistung Heizung, L-Byte, 100% => 1-32767 (600)
REGELABW		EQU 0x60		;Regelabweichung
REGOUT			EQU 0x61		;Regelausgang
REGOUT_DISP		EQU 0x62		;Regelausgang fr Display (0-100%)
P_ANTEIL		EQU 0x63		;P-Anteil
I_ANTEIL_L		EQU 0x64		;I-Anteil, L-Byte
I_ANTEIL_M		EQU 0x65		;I-Anteil, M-Byte
I_ANTEIL_H		EQU 0x66		;I-Anteil, H-Byte
I_LOOP_T2		EQU 0x67		;Schleifenzhler 0,2 sek fr I-Anteil
STAT_PWM		EQU 0x68		;Statusregister PWM:	Bit0 = Regler (PWM) 0= on, 1= off
								;						Bit1 = Bruch Temperaturfhler
								;						Bit2 = Kurzschluss Temperaturfhler
								;						Bit3 = >4s kein Leistungssignal von Zhler MT691
								;						Bit4 = Heizungstemperatur >= Tmax
								;						Bit5 = Heizungstemperatur Begrenzung
								;						Bit6 = Abschalttimer
								;						Bit7 = Funktion Pin8 (RB2): 0= on/off ber Abschalttimer, 1= TX Pb/e 
T_H				EQU 0x69		;Temperatur Heizung
T_MAX			EQU 0x6A		;Max. Temperatur Heizung
TIMER_OFF		EQU 0x6B		;Timer >10s kein Leistungssignal von Zhler MT691
T_ABSCH			EQU 0x6C		;Abschaltzeit, wenn keine Einspeisung mehr erfolgt
TIMER_ABSCH		EQU 0x6D		;Abschaltzeit, wenn keine Einspeisung mehr erfolgt
DISP_NR			EQU 0x6E		;Zhler fr Anzeige auf dem Display: 0=Zb/Pb o. Pe, 1=Ze/Pb o. Pe, 2=Zp/Pp, 3=Uhrzeit/Datum

ZAHL_A0			EQU 0x70		;Rechenregister A, Bit 0-7
ZAHL_A1			EQU 0x71		;Rechenregister A, Bit 8-15
ZAHL_A2			EQU 0x72		;Rechenregister A, Bit 16-23
ZAHL_A3			EQU 0x73		;Rechenregister A, Bit 24-31
ZAHL_B0			EQU 0x74		;Rechenregister B, Bit 0-7
ZAHL_B1			EQU 0x75		;Rechenregister B, Bit 8-15
ZAHL_B2			EQU 0x76		;Rechenregister B, Bit 16-23
ZAHL_B3			EQU 0x77		;Rechenregister B, Bit 24-31
ZAHL_C0			EQU 0x78		;Rechenregister C, Bit 0-7
ZAHL_C1			EQU 0x79		;Rechenregister C, Bit 8-15
ZAHL_C2			EQU 0x7A		;Rechenregister C, Bit 16-23
ZAHL_C3			EQU 0x7B		;Rechenregister C, Bit 24-31
ZAHL_C4			EQU 0x7C		;Rechenregister C, Bit 32-39
ZAHL_C5			EQU 0x7D		;Rechenregister C, Bit 40-47
ZAHL_C6			EQU 0x7E		;Rechenregister C, Bit 48-55
ZAHL_C7			EQU 0x7F		;Rechenregister C, Bit 56-63

;===========================
;  VARIABLEN EEPROM
;===========================

		ORG 	0xF000			;Adresse des EEPROM fr den Brenner 

		de		0xB8			;Adresse 00, EEPROM Adresse fr Speicherung historischer Zhlerstnde (184)
		de		0x32			;Adresse 01, Kp: 0-0,99 (0,5)
		de		0x05			;Adresse 02, Tn: 1-99 (5)
		de		0x02			;Adresse 03, Plast: H-Byte, 100% => 1-9999W (600)
		de		0x58			;Adresse 04, Plast: L-Byte, 100% => 1-9999W (600)
		de		0x8C			;Adresse 05, Tmax Heizung: 0-99C (60C => Tabellenwert: )
		de		0x01			;Adresse 06, Vorteiler Produktionszhler: Imp/kWh x 1000
		de		0x00			;Adresse 07, Regler 0=ein, 1=aus
		de		0x0A			;Adresse 08, Abschaltzeit Heizung wenn keine Einspeisung 1-99min (10min)
		de		0x10			;Adresse 09, Kennzeichnung fr Leistung Byte 1 ( hex: 10 07 01)
		de		0x07			;Adresse 0A, Kennzeichnung fr Leistung Byte 2 ( hex: 10 07 01)
		de		0x00			;Adresse 0B, Kennzeichnung fr Leistung Byte 3 ( hex: 10 07 01)

;__________________________________________________________________________________________

		ORG     0x0000			; Adresse, an die nach einem Reset gesprungen wird
		PAGESEL Start
  		goto    Start           ; Springe zum Programmanfang

		ORG     0x0004          ; Adresse, an die nach einem Interrupt gesprungen wird

		; Ab hier kann der Code fr die Interruptroutinen stehen

		BANKSEL PORTA				;Select BANK 0
		PAGESEL Start				;Select PAGE 0


int_capture						;Ermittlung der Zeit zwischen 2 Impulsen vom Produktionszhler
	btfss PIR3,CCP4IF			;prfen, ob Capture flag gesetzt ist
	goto int_timer1				;Capture flag ist nicht gesetzt, weiter bei nchster Interrupt routine

	movlw d'8'					;Timer 1 L+H zurcksetzen (8 Takte Zeitverzgerung)
	movwf TMR1L
	clrf TMR1H
	movfw TMR1_EXT1				;kopiere Timer 1 Erweiterungs Werte in T_prod_2/3 Register
	movwf T_PROD_2
	movfw TMR1_EXT2
	movwf T_PROD_3
	clrf TMR1_EXT1				;Timer 1 Erweiterungsregister 1+2 zurcksetzen
	clrf TMR1_EXT2
	BANKSEL CCP4CON				;Select BANK 6
	movfw CCPR4L				;Timer 1 L-Byte aus CCPR4L holen und in T_prod_0 Register kopieren
	BANKSEL PORTA				;Select BANK 0
	movwf T_PROD_0
	BANKSEL CCP4CON				;Select BANK 6
	movfw CCPR4H				;Timer 1 H-Byte aus CCPR4H holen und in T_prod_1 Register kopieren
	BANKSEL PORTA				;Select BANK 0
	movwf T_PROD_1

	decfsz ZPROD_PRE			;Vorteiler um 1 reduzieren und prfen ob Null
	goto $+8					;nicht null, weiter
	movfw ZPROD_IMP				;= Null, Rcksetzen des Vorteilers
	movwf ZPROD_PRE
	incfsz ZAEHL_PROD_0			;erhhe 0-Byte des Produktionszhlers um 1 und prfe auf berlauf
	goto $+6					;kein berlauf, weiter
	incfsz ZAEHL_PROD_1			;berlauf, erhhe 1-Byte des Produktionszhlers um 1 und prfe auf berlauf
	goto $+4					;kein berlauf, weiter
	incfsz ZAEHL_PROD_2			;berlauf, erhhe 2-Byte des Produktionszhlers um 1 und prfe auf berlauf
	goto $+2					;kein berlauf, weiter
	incf ZAEHL_PROD_3			;berlauf, erhhe 3-Byte des Produktionszhlers um 1


	bcf PIR3,CCP4IF				;lsche Capture flag
	bsf STAT_DATEN,0			;setze Update Bit 0
	btfss STAT_DATEN,1			;prfe, ob Update Bit 1 gesetzt ist
	goto int_end				;nicht gesetzt, Rcksprung aus dem Interrupt
	bcf STAT_DATEN,1			;lsche Update Bit 1
	movlw 0xFF					;setze Zeit Register auf max Werte
	movwf T_PROD_0
	movwf T_PROD_1
	movwf T_PROD_2
	movwf T_PROD_3
	goto int_end

int_timer1					;Zeitbasis fr Zeit zwischen 2 Impulsen vom Produktionszhler
	btfss PIR1,TMR1IF		;
	goto int_timer2
	bcf PIR1,TMR1IF			;lsche Flag
	incfsz TMR1_EXT1		;erhhe Timer1 Erweiterungsregister um 1 und prfe ob 0
	goto int_end			;Sekunde noch nicht abgelaufen, 
	incfsz TMR1_EXT2		;erhhe Timer1 Erweiterungsregister um 1 und prfe ob 0
	goto int_end			;Sekunde noch nicht abgelaufen, 
	bsf STAT_DATEN,1
	goto int_end

int_timer2					;Zeitbasis fr interne Uhr und zeitabhngige Status Bits
	btfss PIR1,TMR2IF		;prfen, ob Timer2 flag gesetzt ist
	goto int_rx				;flag ist nicht gesetzt, weiter bei nchster Prfung
	bcf PIR1,TMR2IF			;flag ist gesetzt, lsche flag

	btfsc STAT_TIMER,0		;prfen, ob Bit0 nicht gesetzt ist =>8ms Timer fr Beendigung Datenbertragung
	bsf STAT_TIMER,1		;Bit0 ist gesetzt, setze auch Bit1, Wenn Bit0+Bit1 gesetzt sind, wurde mindestens 8ms kein RX Byte empfangen
	bsf STAT_TIMER,0		;setze Bit0

	decfsz I_LOOP_T2		;reduziere Timer fr I-Anteil (32 x 250 x 25 = 200.000 Takte => 0,2 sek) um 1 und prfe ob 0
	goto $+4				;Timer fr I-Anteil noch nicht 0, weiter
	movlw d'25'				;Timer fr I-Anteil 0, rcksetzen mit Wert 25
	movwf I_LOOP_T2
	bsf STAT_TIMER,6		;0,2 Sekunden abgelaufen, setze Status Bit fr Neuberechnung I-Anteil
	decfsz TMR2_EXT			;reduziere Timer2 Erweiterung (32 x 250 x 125 = 1.000.000 Takte => 1,0 sek) um 1 und prfe ob 0
	goto int_end			;Sekunde noch nicht abgelaufen, Rcksprung aus dem Interrupt
	movlw d'125'			;Sekunde abgelaufen, rcksetzen Timer2 Erweiterung mit Wert 125
	movwf TMR2_EXT		

	bsf STAT_TIMER,2		;Sekunde abgelaufen, setze Statusbit fr update Display
	incf TIMER_OFF			;erhhe Abschalttimer um 1 Sekunde
	movlw d'11'				;prfen, ob Abschalttimer >10 Sekunden
	subwf TIMER_OFF,w
	btfsc STATUS,C
	bsf STAT_PWM,3			;Abschalttimer >10 Sekunden, setze Abschalt Bit
	incf SEKUNDE			;Erhhe Sekundenzhler um 1
	btfsc SEKUNDE,0			;Prfen der Bits 0+1 fr 4 Sekunden 
	goto $+5
	btfsc SEKUNDE,1
	goto $+3
	bsf STAT_TIMER,3		;Bits 0+1 = 00, d.h. 4 Sekunden sind abgelaufen, Setze Statusbits fr 4s (Display Umschaltung)
	bsf STAT_TIMER,4

	movlw d'60'				;hole Wert 60 nach W
	subwf SEKUNDE,w			;und subtrahiere die Sekunden
	btfss STATUS,Z			;prfen, ob Ergebniss =0, d.h. Sekundenzhler bergelaufen
	goto int_end			;Sekundenzhler noch nicht bergelaufen
	clrf SEKUNDE			;Sekundenzhler bergelaufen, rcksetzen mit Wert 0


	btfsc STAT_PWM,0		;prfen, ob Bit fr "Regler Aus" nicht gesetzt ist
	goto $+0x0A				;Bit fr "Regler Aus" ist gesetzt, berspringe die nchsten 2 Zeilen
	btfss OUT_ON_OFF		;prfen,ob Ausgang gesetzt ist
	goto $+8
	incf TIMER_ABSCH		;erhhe Zhler um 1
	movfw T_ABSCH			;Timer Abschaltung - Abschaltzeit 
	subwf TIMER_ABSCH,w
	btfss STATUS,C			;prfen, ob 0 oder positiv, d.h. Abschaltzeit erreicht oder berschritten
	goto $+3				;negativ, d.h Abschaltzeit noch nicht erreicht, weiter
	bcf OUT_ON_OFF			;0 oder positiv, d.h. Abschaltzeit erreicht oder berschritten, setze Ausgang auf low
	bsf STAT_PWM,6			;setze Statusbit

	incf MINUTE				;Erhhe Minutenzhler um 1
	movlw d'60'				;hole Wert 60 nach W
	subwf MINUTE,w			;und subtrahiere die Minuten
	btfss STATUS,Z			;prfen, ob Ergebniss =0, d.h. Minutenzhler bergelaufen
	goto int_end			;Minutenzhler noch nicht bergelaufen
	clrf MINUTE				;Minutenzhler bergelaufen, rcksetzen mit Wert 0

	incf STUNDE				;Erhhe Stundenzhler um 1
	movlw d'24'				;hole Wert 24 nach W
	subwf STUNDE,w			;und subtrahiere die Stunden
	btfss STATUS,Z			;prfen, ob Ergebniss =0, d.h. Stundenzhler bergelaufen
	goto int_end			;Stundenzhler noch nicht bergelaufen
	clrf STUNDE				;Stundenzhler bergelaufen, rcksetzen mit Wert 0

	bsf STAT_TIMER,5		;setze Statusbit fr neuen Tag
	bsf PCLATH,3			;Select Page 1	
	call inc_datum			;Unterprogramm fr neuen Tag/Monat/Jahr
	bcf PCLATH,3			;Select Page 0
	goto int_end

int_rx						;Speichern des Datensatzes vom Zhler eHZ MT691 in den Registern von Bank 1,2+3
	btfss PIR1,RCIF			;
	goto int_end
	movfw FSR_REG_0H		;kopiere gespeicherten FSR0H Wert (indirekte Adressierung Bank0/1 oder Bank2/3)
	movwf FSR0H
	movlw 0x02				;FSR1H auf 2 setzen (indirekte Adressierung Bank4/5)
	movwf FSR1H
	movfw FSR_REG_0			;kopiere gespeicherte FSR Werte 
	movwf FSR0
	movfw FSR_REG_1
	movwf FSR1
	BANKSEL ANSELA			;Select BANK 3
	movfw RCREG				;hole Rx Wert nach W
	BANKSEL PORTA			;Select BANK 0
	movwf INDF0

	clrf TIMER_OFF			;lsche Abschalttimer PWM
	bcf STAT_PWM,3			;lsche Statusbit fr Abschalttimer PWM
	bcf STAT_TIMER,0		;lsche Stat_timer Bits 0+1 (Beendigung der Datenbertragung)
	bcf STAT_TIMER,1
	bsf STAT_DATEN,2		;setze Bit (Datenbertragung nicht beendet)
	btfss FSR_REG_0H,0		;Abfrage, ob letztes Datenregister erreicht ist(Bank3, Register 0xEF)
	goto $+5
	movlw 0xEF
	subwf FSR_REG_0,w
	btfsc STATUS,Z
	goto int_end			;letztes Datenregister erreicht, Rcksprung
	movlw 0x28				;Abfrage, ob 8 aufeinanderfolgende Bytes der Sequenz  77 07 01 00 01 08 00 FF entsprechen,
	subwf FSR_REG_1,w		;  die in Bank4 in den Registern 0x20 bis 0x27 abgelegt ist
	btfsc STATUS,Z
	goto int_rx_daten		;ja, Daten entsprechen der Sequenz, weiter bei int rx daten (Erhhung der Register Adresse)
	movfw INDF1				;nein, Daten entsprechen noch nicht der Sequenz, Prfung des aktuellen empfangenen Byte
	subwf INDF0,w
	btfss STATUS,Z
	goto int_rx_reset		;Daten sind nicht gleich, Rcksetzen der FSR_REG Register
	incf FSR_REG_0			;Daten sind gleich, Erhhen der FSR_REG Register
	incf FSR_REG_1
	goto int_end

int_rx_reset				;Daten sind nicht gleich, Rcksetzen der FSR_REG Register
	movlw 0xA0
	movwf FSR_REG_0
	movlw 0x20
	movwf FSR_REG_1
	goto int_end

int_rx_daten
	incf FSR_REG_0
	btfsc FSR_REG_0H,0		;prfen, ob Bank 0/1 (=>0) oder Bank 2/3 (=>1) gewhlt ist
	goto int_rx_daten_b23	;Bank 2/3 (=>1) gewhlt

	movlw 0xF0				;Bank 0/1 (=>0) gewhlt, prfen, ob max. Speicherplatz +1 erreicht ist (Bank 1, 0xF0)
	subwf FSR_REG_0,w
	btfss STATUS,Z
	goto int_end			;nicht erreicht, Rcksprung
	bsf FSR_REG_0H,0		;max. Speicherplatz +1 ist erreicht (Bank 1, 0xF0), setze Bit fr Bank 2/3
	movlw 0x20				;und setze FSR 0 auf die Startadresse 0x20 (Bank 2)
	movwf FSR_REG_0
	goto int_end			;Rcksprung

int_rx_daten_b23			;Bank 2/3 (=>1) gewhlt
	movlw 0x70				;prfen, ob max. Speicherplatz +1 erreicht ist (Bank 2, 0x70)
	subwf FSR_REG_0,w
	btfss STATUS,Z
	goto int_end			;nicht erreicht
	movlw 0xA0				;max. Speicherplatz +1 Bank 2 ist erreicht 
	movwf FSR_REG_0			;setze FSR 0 auf die Startadresse 0xA0 (Bank 3)

int_end

		retfie                    	;Rcksprung vom Interrupt

		

;__________________________________________________________________________________________

Start						; Beginn des Programms
	; Initialisierungen
	BANKSEL PORTA			;Select BANK 0
	clrf PORTA				;Lsche alle Ausgangsports
	clrf PORTB
	clrf INTCON				;alle Interrupts verbieten

	BANKSEL LATA			;Select BANK 2
	clrf LATA
	clrf LATB
	clrf APFCON0			;Bit0(CCP1SEL): 0=PWM auf RB3,1=PWM auf RB0
	bsf APFCON0,CCP1SEL		;PWM auf RB0

	BANKSEL ANSELA			;Select BANK 3
	clrf ANSELA				;Alle Pins von Port A sind digitale I/O Ports
	movlw b'00001000'		;Pin RB3 ist analoger Eingang (AN9)
	movwf ANSELB

	BANKSEL TRISA			;Select BANK 1
	movlw B'00001000'		;pull-up, internen Takt zhlen, bit 3: TMR0 Vorteiler 1:1
	movwf OPTION_REG

	movlw b'11110000'		;setze RA0-RA3 als Ausgang, den Rest als Eingang
	movwf TRISA
	movlw b'11111010'		;setze RB0 und RB2 als Ausgang, den Rest als Eingang
	movwf TRISB
	movlw b'01010000'		;bit 7=0: linksbndig, bit 6-4=101: f/16, bit 2:-Vref =Vss, bit 1-0:+Vref =Vdd
	movwf ADCON1
	movlw b'00100100'		;Channel 9 gewhlt (AN9)
	movwf ADCON0			;

	BANKSEL ANSELA			;Select BANK 3
	clrf BAUDCON			;Baudrate Soll 9600
	movlw d'25'
	movwf SPBRGL
	bsf TXSTA,BRGH			;High Baud Rate Select bit(x4)
	bsf RCSTA,CREN			;Enables receiver
	bsf RCSTA,SPEN			;Serial port enabled

	BANKSEL WPUA			;Select BANK 4
	movlw b'00100000'		;Pull-Up Port A
	movwf WPUA
	movlw b'11110110'		;Pull-Up Port B
	movwf WPUB				

	BANKSEL CCPR1L			;Select BANK 5
	movlw b'00000001'		;
	movwf CCPTMRS			;CCP1 verwendet Timer 4
	movlw b'00010001'		;Bits 1-0: 01: A ist aktiv, B inaktiv,  10: A ist inaktiv, B aktiv
	movwf PSTR1CON
	clrf CCPR1L				; 0%
	clrf CCPR1H				; 0%
	BANKSEL T4CON			;Select BANK 8
	clrf T4CON				;prescaler 1:1, postscaler 1:1
	movlw 0xFF				;Frequenz auf 3,9 kHz einstellen: 4 MHz / 4 / (255+1) = 3,9 KHz
	movwf PR4				;

	BANKSEL CCP4CON			;Select BANK 6
	movlw b'00000100'		;Capture mode, every falling edge
	movwf CCP4CON
	
	BANKSEL PORTA			;Select BANK 0

	clrf FSR0H				;lsche IRP Bit (indirekte Adressierung Bank0/1)
	movlw 0x20				;Register 0x20 bis 0x7F lschen
	movwf FSR0
	clrf INDF0				;lsche INDF0 Register
	incf FSR0,F 			;Erhhe FSR0 Register
	movlw 0x80				;maximale adresse +1
	subwf FSR0,w			;subtrahiere FSR0 in w
	btfss STATUS,Z			;prfen, ob 0, d.h. maximale adresse +1 erreicht ist
	goto $-5				;maximale adresse +1 noch nicht erreicht, zurck zur Schleife

	bsf PCLATH,3			;Select Page 1
	call read_eeprom_data	;lesen der EEPROM Daten und schreiben in die F Register
	call lade_zaehl_dat
	call init_display		;Initialisierung des Displays
	bcf PCLATH,3			;Select Page 0

	bsf STAT_TIMER,0
	bsf STAT_DATEN,1
	bsf STAT_PWM,6			;setze Statusbit Abschalttimer
	movlw d'25'				;lade Timer2 Zhler mit Wert 25
	movwf I_LOOP_T2

	movlw 0x01
	movwf TAG
	movwf MONAT
	movlw d'23'
	movwf JAHR

	BANKSEL TRISA			;Select BANK 1
	movlw b'00100011'		;Interrupts: Bit5: RCIE (USART Receive), Bit1: TMR2IE, Bit0: TMR1IE,
	movwf PIE1				;
	bsf PIE3,CCP4IE			;CCP4 Interrupt erlauben
	BANKSEL PORTA			;Select BANK 0
	bcf PIR3,CCP4IF			;lsche CCP4 flag

	bsf PCLATH,3			;Select Page 1
	call disp_start
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	call wait_250_p0		;Wartezeit von 250ms Page0
	call wait_250_p0		;Wartezeit von 250ms Page0
	call wait_250_p0		;Wartezeit von 250ms Page0
	call clear_reg
	movlw d'125'
	movwf TMR2_EXT
	movlw d'249'
	movwf PR2
	movlw b'00001110'		;Timer 2: postscaler = 2, prescaler = 16, timer 2 on
	movwf T2CON
	bsf T1CON,TMR1ON		;Timer 1 einschalten
	bcf PIR1,TMR1IF			;lsche Timer 1 flag
	bsf INTCON,PEIE
	bsf INTCON,GIE

;Testbereich_Anfang

;Testbereich_Ende

main						;Beginn der Hauptschleife
	call befehl				;Aufruf Unterprogramm fr Tastatur-/IR-Befehle
	btfsc BEFEHL,0			;prfen, ob Menue Taste gedrckt wurde
	goto menue				;Menue Taste wurde gedrckt, Weiter bei Programmteil menue
	btfss BEFEHL,1			;prfen, ob OK Taste gedrckt wurde
	goto $+3
	bcf STAT_DATEN,4		;lsche Bit fr manuelle Displayanzeige
	call wait_250_p0		;Wartezeit von 250ms Page0
	btfsc BEFEHL,2			;prfen, ob Up Taste gedrckt wurde
	call disp_inc_man		;Unterprogramm zur manuellen Erhhung der Displayanzeige
	btfsc BEFEHL,3			;prfen, ob Rechts Taste gedrckt wurde
	call disp_dec_man		;Unterprogramm zur manuellen Erhhung der Displayanzeige
	btfsc STAT_TIMER,5		;Prfen ob Bit fr neuen Tag nicht gesetzt ist
	call store_z_vt			;Bit ist gesetzt, Aufruf Unterprogramm zum Speichern der Zhlerstnde des Vortages/Vormonat

	btfsc STAT_PWM,0		;prfen, ob Bit fr "Regler Aus" nicht gesetzt ist
	goto $+3				;Bit fr "Regler Aus" ist gesetzt, berspringe die nchsten 2 Zeilen
	btfsc STAT_TIMER,6		;prfen, ob 200ms Timer Bit gesetzt ist
	call update_reg_i		;Temperatur messen und I-Anteil Regler berechnen

	btfsc STAT_DATEN,4		;prfen, ob Bit fr manuelle Displayanzeige nicht gesetzt ist
	goto $+3				;Bit ist gesetzt, berspringe die automatische Erhhung der Displayanzeige
	btfsc STAT_TIMER,3		;prfen, ob 4s Bit fr automatische Display Umschaltung nicht gesetzt ist
	call disp_inc_auto
	btfsc STAT_TIMER,4		;prfen, ob 4s Bit fr berprfung der Leistung Produktion nicht gesetzt ist
	call test_p_prod

	btfsc STAT_DATEN,0		;prfen, ob neuer Impuls von Produktionszhler detektiert wurde (1000 Imp/kWh)
	call produktion

	btfss STAT_TIMER,1		;prfen, ob >8ms kein Daten vom USART empfangen wurden, d.h. bertragung beendet
	goto $+3				;8ms noch nicht abgelaufen, weiter
	btfsc STAT_DATEN,2		;8ms abgelaufen, prfe ob Update Bit nicht gesetzt ist
	call auswert_rx			;Update Bit ist gesetzt, Auswertung der Daten	

	movfw STAT_PWM			;prfen, ob keine Fehlerbits gesetzt sind, d.h =0
	movwf TEMP_11
	bcf TEMP_11,7
	movfw TEMP_11	
	btfss STATUS,Z
	call pwm_off			;nicht null, d.h. Fehler => PWM abschalten
	btfss STAT_TIMER,2		;prfen, ob das Update bit fr das Display gesetzt ist (1 sek, neuer Impuls, neue Auswertung von eHZ MT691)
	goto main				;keine neuen Daten, Zurck zur Hauptprogrammschleife
	bsf PCLATH,3			;Select Page 1
	call disp_main
	bcf PCLATH,3			;Select Page 0
	goto main				;zurck zur Hauptprogrammschleife


menue						;Programmteil Men
	call pwm_off			;sicherheitshalber PWM ausschalten
	movlw d'1'				;rcksetzen des Menzhlers auf1
	movwf MENU_COUNT
	movwf M_HOR_CNT
	bsf MENU_STAT,7			;Setze Bit fr update Men
	call wait_250_p0		;Wartezeit von 250ms Page0
menue_loop
	bsf PCLATH,3			;Select Page 1
	btfsc MENU_STAT,7		;prfen des Update Bits
	call disp_menue_1		;Bit ist gesetzt, Update der Anzeige fr Hauptmen
	bcf PCLATH,3			;Select Page 0
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-Befehle
	btfsc BEFEHL,3			;prfen, ob Rechts Taste gedrckt wurde
	call menue_dn			;Taste gedrckt, Aufruf Unterprogramm Men down
	btfsc BEFEHL,2			;prfen, ob Up Taste gedrckt wurde
	call menue_up			;Taste gedrckt, Aufruf Unterprogramm Men up
	btfsc BEFEHL,0			;prfen, ob Men Taste gedrckt wurde
	goto menue_end			;Taste gedrckt, weiter bei Men ende
	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist
	goto menue_loop			;Taste nicht gedrckt, zurck zur Schleife

	call wait_250_p0		;Wartezeit von 250ms Page0
	bsf PCLATH,3			;Select Page 1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	bcf PCLATH,3			;Select Page 0

	movlw 0x07				;Programmzhler fuer menuetabelle im Bereich 0x0700..0x07FF 
	movwf PCLATH
	movfw MENU_COUNT
	goto menue_tab

menu_zalt_set				;Menpunkt 1.Anzeige historischer Zhlerstnde: letzte 5 Tage und Vormonat
	clrf TEMP_11			;0= Tag-1...13=Tag-14, 14=Vormonat
	bsf PCLATH,3			;Select Page 1
	call disp_set_zalt		;
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
loop_zalt_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto menu_zalt1_set		;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "up" nicht gedrckt ist
	call inc_zalt			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call dec_zalt			;wenn ja berspringe diesen Befehl
	goto loop_zalt_set		;zurck zur Schleife

menu_zalt1_set
	clrf TEMP_12			;lsche Register (0 = Zb/Ze , 1 = Zp)
	movlw d'14'				;prfen ob Zhler = 14 (Vormonat)
	subwf TEMP_11,w
	btfss STATUS,Z
	goto $+4				;Zhler nicht 14
	movlw d'16'				;Zhler = 14, EEPROM Adresse fr Vormonat = 16
	movwf TEMP_11
	goto menu_zalt1_set1

	movfw TEMP_11			;x12
	movwf TEMP_13
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf TEMP_13				;3x nach links schieben => x8
	rlf TEMP_13
	rlf TEMP_13
	movfw TEMP_11			;4x addieren => x12
	addwf TEMP_13
	movfw TEMP_11
	addwf TEMP_13
	movfw TEMP_11
	addwf TEMP_13
	movfw TEMP_11
	addwf TEMP_13

	movfw EEPROM_ADR		;EEPROM Adresse fr Speicherung historischer Zhlerstnde (184)
	movwf TEMP_11
	movlw d'28'				;offset subtrahieren
	subwf TEMP_11
	movfw TEMP_13
	subwf TEMP_11
	movlw d'168'
	btfss STATUS,C
	addwf TEMP_11
	movlw d'28'				;offset addieren
	addwf TEMP_11
menu_zalt1_set1
	bsf PCLATH,3			;Select Page 1
	call disp_set_zalt1		;
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
loop_zalt1_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "up" nicht gedrckt ist
	call inc_zalt1			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_zalt1			;wenn ja berspringe diesen Befehl
	goto loop_zalt1_set		;zurck zur Schleife
	return

menu_zp_set					;Menpunkt 2.Zhler Produktion
	movlw d'9'				;Anzeige hat 9 Stellen, die gesetzt werden
	movwf M_HOR_CNT_MAX		;
	movfw ZAEHL_PROD_0		;Umwandeln des 24 Bit Prod Zhler Register in die einzelnen 8 Digits
	movwf ZAHL_C0
	movfw ZAEHL_PROD_1		;
	movwf ZAHL_C1
	movfw ZAEHL_PROD_2		;
	movwf ZAHL_C2
	movfw ZAEHL_PROD_3		;
	movwf ZAHL_C3
	call convert_hex_dig	;Unterprogramm zum Zerlegen einer 24-Bit Zahl in die einzelnen Digits (max.999.999)zur Anzeige auf dem Display
	movlw 0x06				;EEPROM Adresse gespeicherter Wert Produktionszhler Imp/kWh
	bsf PCLATH,3			;Select Page 1
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bcf PCLATH,3			;Select Page 0
	movwf TEMP_11
	bsf MENU_STAT,0			;setze Statusbit
	call wait_250_p0		;Wartezeit von 250ms Page0	
	bsf PCLATH,3			;Select Page 1
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bcf PCLATH,3			;Select Page 0
loop_zp_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto copy_zp			;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_m_hor_cnt		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1	
	btfsc MENU_STAT,0		;prfen ob bit in register nicht gesetzt ist
	call dig_zp_end			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "up" nicht gedrckt ist
	call dig_zp				;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	goto loop_zp_set		;zurck zur Schleife
copy_zp	
	call convert_dig_hex	;Unterprogramm zum Umwandeln der einzelnen Digits (max.16.777.216) in eine 24-Bit Zahl
	movfw ZAHL_C0			;kopiere Register in Prod. Zhler Register
	movwf ZAEHL_PROD_0
	movfw ZAHL_C1			;
	movwf ZAEHL_PROD_1
	movfw ZAHL_C2			;
	movwf ZAEHL_PROD_2
	movfw ZAHL_C3			;
	movwf ZAEHL_PROD_3
	movfw TEMP_11			;Wert fr Impulse/kWh (x1000)
	movwf TEMP_13			;Speichern des Wertes fr Impulse/kWh (x1000) im internen EEPROM
	movlw 0x06				;EEPROM Adresse
	bsf PCLATH,3			;Select Page 1
	call writeEE
	bcf PCLATH,3			;Select Page 0
	movfw TEMP_11			;Wert fr Impulse/kWh (x1000)
	bsf PCLATH,3			;Select Page 1
	call mul_10				;mit 10 multiplizieren
	bcf PCLATH,3			;Select Page 0
	movwf ZPROD_PRE			;Vorteiler fr Produktionszhler
	movwf ZPROD_IMP			;Produktionszhler Impulse/kWh (x100)
	goto menue_end			;gehe zum Ende des Programmteils

menu_datuz_set				;Menpunkt 3.Datum/Uhrzeit
	movlw d'10'				;Anzeige hat 10 Stellen, die gesetzt werden
	movwf M_HOR_CNT_MAX		;
	bsf PCLATH,3			;Select Page 1
	call convert_datuz		;Unterprogramm zum Zerlegen der Werte fr Tag, Monat, Jahr, Stunde und Minute in die einzelnen Digits
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bcf PCLATH,3			;Select Page 0
	bsf MENU_STAT,0			;setze Statusbit
	call wait_250_p0		;Wartezeit von 250ms Page0	
loop_datuz_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto copy_datuz			;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_m_hor_cnt		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1	
	btfsc MENU_STAT,0		;prfen ob bit in register nicht gesetzt ist
	call dig_datuz_end		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "up" nicht gedrckt ist
	call dig_datuz			;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0	
	goto loop_datuz_set		;zurck zur Schleife
copy_datuz	
	bsf PCLATH,3			;Select Page 1
	call reconv_datuz		;Unterprogramm zum kombinieren der Digits zu den Werten fr Tag, Monat, Jahr, Stunde und Minute
	bcf PCLATH,3			;Select Page 0
	goto menue_end



menu_testrx_set				;Menpunkt 4.Zhler auslesen
	bsf MENU_STAT,0			;setze update bit
	bcf MENU_STAT,1			;0=bytes 0-159; 1=bytes 160-319
testrx_loop
	bsf PCLATH,3			;Select Page 1
	btfsc MENU_STAT,0		;prfen, ob Wert sich nicht gendert hat
	call disp_set_testrx	;Wert hat sich gendert, Anzeigen des neuen Kontrast Wertes
	bcf PCLATH,3			;Select Page 0

	call befehl				;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfsc BEFEHL,0			;prfen, ob Men Taste nicht gedrckt ist
	goto testrx_set_end1
	btfsc BEFEHL,1			;prfen, ob OK Taste nicht gedrckt ist
	goto testrx_set_end
	btfsc BEFEHL,2			;prfen, ob Up Taste nicht gedrckt ist
	call incdec_testrx
	btfsc BEFEHL,3			;prfen, ob Rechts Taste nicht gedrckt ist
	call incdec_testrx
	goto testrx_loop

testrx_set_end
	call wait_250_p0		;Wartezeit von 250ms Page0	
	BANKSEL TRISA			;Select BANK 1
	bcf PIE1,RCIE			;USART Receive Interrupt verbieten
	BANKSEL ANSELA			;Select BANK 3
	bcf RCSTA,CREN			;disables receiver
	BANKSEL PORTA			;Select BANK 0
testrx_end_loop
	btfss IN_MENUE			;prfen, ob Set Taste gedrckt ist,
	goto testrx_set_end1
	btfsc STAT_TIMER,1		;prfen, ob >8ms kein Daten vom USART empfangen wurden, d.h. bertragung beendet
	goto $+6				;Bit ist gesetzt, d.h. keine Daten vom USART, => bertragung beendet
	btfsc IN_RX				;prfen, ob Daten gesendet werden
	goto testrx_end_loop	;keine Daten, zurck zur Schleife
	bcf STAT_TIMER,0		;Daten werden noch gesendet, lsche 8ms Timer Bits 0/1
	bcf STAT_TIMER,1
	goto testrx_end_loop	;zurck zur Schleife

	call clear_reg			;lsche Register und setze FSR0/FSR0H Startdaten
	movlw d'240'			;vorbereiten, falls bytes 240-479 gewhlt sind
	movwf LOOP_1
	bcf STAT_TIMER,0		;lsche 8ms Timer Bits 0/1
	bcf STAT_TIMER,1
	bcf STAT_DATEN,2		;lsche update Bit
	bcf STAT_DATEN,3		;lsche berlaufbit Speicherbereich
	BANKSEL ANSELA			;Select BANK 3
	bsf RCSTA,CREN			;disables receiver
	BANKSEL PORTA			;Select BANK 0

	btfss MENU_STAT,1		;prfen, ob Bytes 240-479 gewhlt sind
	goto testrx_set_dat

testrx_end_loop1			;Bytes 240-479 sind gewhlt
	btfss IN_MENUE			;prfen, ob Set Taste gedrckt ist,
	goto testrx_set_end1
	btfss STAT_TIMER,1		;prfen, ob >8ms kein Daten vom USART empfangen wurden, d.h. bertragung beendet
	goto $+3				;8ms noch nicht abgelaufen, weiter
	btfsc STAT_DATEN,2		;8ms abgelaufen, prfe ob Update Bit nicht gesetzt ist
	goto testrx_set_ausw
	btfss PIR1,RCIF
	goto testrx_end_loop1
	BANKSEL ANSELA			;Select BANK 3
	movfw RCREG
	BANKSEL PORTA			;Select BANK 0
	bcf STAT_TIMER,0
	bcf STAT_TIMER,1
	bsf STAT_DATEN,2
	decfsz LOOP_1
	goto testrx_end_loop1

testrx_set_dat				;Bytes 0-239 sind gewhlt
	btfss IN_MENUE			;prfen, ob Set Taste gedrckt ist,
	goto testrx_set_end1
	btfss STAT_TIMER,1		;prfen, ob >8ms kein Daten vom USART empfangen wurden, d.h. bertragung beendet
	goto $+3				;8ms noch nicht abgelaufen, weiter
	btfsc STAT_DATEN,2		;8ms abgelaufen, prfe ob Update Bit nicht gesetzt ist
	goto testrx_set_ausw
	btfss PIR1,RCIF
	goto testrx_set_dat
	BANKSEL ANSELA			;Select BANK 3
	movfw RCREG				;hole empfangenes Byte nach W
	BANKSEL PORTA			;Select BANK 0
	movwi FSR0++			;schiebe Wert nach INDF0 und erhhe FSR0 um 1 
	bcf STAT_TIMER,0		;lsche Timer 2 Bits 0+1 (8ms)
	bcf STAT_TIMER,1
	bsf STAT_DATEN,2
	call set_fsr0_reg
	btfss STAT_DATEN,3
	goto testrx_set_dat

testrx_set_ausw
	BANKSEL ANSELA			;Select BANK 3
	bcf RCSTA,CREN			;disables receiver
	BANKSEL PORTA			;Select BANK 0
	clrf TEMP_11
	bsf MENU_STAT,0			;setze bit fr vernderung des wertes
testrx_ausw_loop
	bsf PCLATH,3			;Select Page 1
	btfsc MENU_STAT,0		;prfen, ob Wert sich nicht gendert hat
	call disp_ausw_testrx	;Wert hat sich gendert, Anzeigen des neuen Kontrast Wertes
	bcf PCLATH,3			;Select Page 0
	call befehl				;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfsc BEFEHL,0			;prfen, ob Men Taste nicht gedrckt ist
	goto testrx_set_end1
	btfsc BEFEHL,1			;prfen, ob OK Taste nicht gedrckt ist
	goto testrx_set_end1
	btfsc BEFEHL,2			;prfen, ob Up Taste nicht gedrckt ist
	call inc_testrx
	btfsc BEFEHL,3			;prfen, ob Rechts Taste nicht gedrckt ist
	call dec_testrx
	goto testrx_ausw_loop

testrx_set_end1
	bcf STAT_DATEN,2		;lsche Statusbit fr Ende bertragung
	bcf STAT_DATEN,3		;lsche Statusbit berlauf
	call clear_reg			;lsche Register
	BANKSEL ANSELA			;Select BANK 3
	bsf RCSTA,CREN			;Enables receiver
	BANKSEL TRISA			;Select BANK 1
	bsf PIE1,RCIE			;USART Receive Interrupt erlauben
	BANKSEL PORTA			;Select BANK 0
	goto menue_end

menu_regpara_set			;Menpunkt 5.Einstellung der Reglerparameter
	movlw d'13'				;Anzeige hat 13 Stellen, die gesetzt werden
	movwf M_HOR_CNT_MAX		;
	bsf PCLATH,3			;Select Page 1
	call convert_regpara	;Unterprogramm zum Zerlegen der Werte fr Kp, Tn und P last in die einzelnen Digits
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bcf PCLATH,3			;Select Page 0
	bsf MENU_STAT,0			;setze Statusbit
	call wait_250_p0		;Wartezeit von 250ms Page0	
loop_regpara_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto copy_regpara		;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_m_hor_cnt		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1	
	btfsc MENU_STAT,0		;prfen ob bit in register nicht gesetzt ist
	call dig_regpara_end	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "up" nicht gedrckt ist
	call dig_regpara		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0	
	goto loop_regpara_set	;zurck zur Schleife
copy_regpara	
	bsf PCLATH,3			;Select Page 1
	call reconv_regpara		;Unterprogramm zum kombinieren der Digits zu den Werten fr Kp, Tn und P last
	call store_regpara		;Unterprogramm zum Speichern der Regler Parameter im internen EEPROM
	bcf PCLATH,3			;Select Page 0
	goto menue_end			;gehe zum Ende des Programmteils


menu_testpwm_set			;Menpunkt 6: Test des PWM Ausgangs
	bsf MENU_STAT,0
	clrf REGOUT_DISP
testpwm_set_loop
	bsf PCLATH,3			;Select Page 1
	btfsc MENU_STAT,0		;prfen, ob Wert sich nicht gendert hat
	call disp_set_testpwm	;Wert hat sich gendert, Anzeigen des neuen Ausgangswertes
	bcf PCLATH,3			;Select Page 0
	call befehl				;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfsc BEFEHL,0			;prfen, ob Men Taste nicht gedrckt ist
	goto testpwm_set_end
	btfsc BEFEHL,1			;prfen, ob OK Taste nicht gedrckt ist
	goto testpwm_set_end
	btfsc BEFEHL,2			;prfen, ob Up Taste nicht gedrckt ist
	call inc_testpwm
	btfsc BEFEHL,3			;prfen, ob Rechts Taste nicht gedrckt ist
	call dec_testpwm
	goto testpwm_set_loop
testpwm_set_end				;Menpunkt beenden
	call pwm_off			;Regler zurcksetzen
	goto menue_end

menu_p_kenn_set				;Menpunkt 7.setzen der Kennung fr Leistung (0x10 0x07 0x00 )
	movlw d'6'				;Anzeige hat 6 Stellen, die gesetzt werden
	movwf M_HOR_CNT_MAX		;
	bsf PCLATH,3			;Select Page 1
	call convert_p_kenn		;Unterprogramm zum Zerlegen der Werte fr die 3 Byte der Leistungskennung in die einzelnen Digits
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	bcf PCLATH,3			;Select Page 0
	bsf MENU_STAT,0			;setze Statusbit
	call wait_250_p0		;Wartezeit von 250ms Page0	
loop_p_kenn_set
	call befehl				;Aufruf Unterprogramm zum laden der Tasten-/IR-Befehle
	btfsc BEFEHL,0			;prfen ob Taster fr Befehl Menue nicht gedrckt ist
	goto menue_end			;Taste ist gedrckt, Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;prfen ob Taster fr Befehl OK nicht gedrckt ist
	goto copy_p_kenn		;Taste ist gedrckt, gehe zum Ende des Programmteils
	btfsc BEFEHL,3			;prfen ob Taster fr Befehl "right" nicht gedrckt ist
	call inc_m_hor_cnt		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1	
	btfsc MENU_STAT,0		;prfen ob bit in register nicht gesetzt ist
	call dig_p_kenn_end		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;prfen ob Taster fr Befehl "up" nicht gedrckt ist
	call dig_p_kenn			;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0	
	goto loop_p_kenn_set	;zurck zur Schleife
copy_p_kenn	
	bsf PCLATH,3			;Select Page 1
	call reconv_p_kenn		;Unterprogramm zum kombinieren der Digits zu den 3 Byte der Leistungskennung
	bcf PCLATH,3			;Select Page 0
	goto menue_end


menu_funct_rb2_set			;Menpunkt 8: Funktion des Ausgangs RB2 (Pin8): Ausgang On/Off oder Ausgang Tx Pb/e
	bsf MENU_STAT,0			;setze Statusbit
	bsf PCLATH,3			;Select Page 1
	call convert_stat_pwm	;kopiere den Status von Stat_PWM Bit 0+7 nach Menu_Stat Bit 2+3, Bit 2 wird im Men verwendet
	bcf PCLATH,3			;Select Page 0
frb2_set_loop
	bsf PCLATH,3			;Select Page 1
	btfsc MENU_STAT,0		;prfen, ob Wert sich nicht gendert hat
	call disp_set_frb2		;Wert hat sich gendert, Anzeigen des neuen Ausgangswertes
	bcf PCLATH,3			;Select Page 0
	call befehl				;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfsc BEFEHL,0			;prfen, ob Men Taste nicht gedrckt ist
	goto menue_end
	btfsc BEFEHL,1			;prfen, ob OK Taste nicht gedrckt ist
	goto frb2_set_end
	btfsc BEFEHL,2			;prfen, ob Up Taste nicht gedrckt ist
	call incdec_frb2
	btfsc BEFEHL,3			;prfen, ob Rechts Taste nicht gedrckt ist
	call incdec_frb2
	goto frb2_set_loop
frb2_set_end				;Menpunkt beenden
	bsf PCLATH,3			;Select Page 1
	call reconv_stat_pwm	;kopiere den Status von Menu_Stat Bit 2+3 nach Stat_PWM Bit 0+7
	call store_stat_pwm		;speichern von Stat_Pwm im internen EEPROM
	call tx_on_off			;Unterprogramm zum Ein-/Ausschalten des TX Ausgangs 
	bcf PCLATH,3			;Select Page 0
	goto menue_end


menue_end					;Standardroutine fr Beendigung des Mens
	movlw 0xA0				;lade FSR0 Register mit 0xA0 und FSR0H Register mit 0x00
	movwf FSR0
	clrf FSR0H
	clrf MENU_STAT			;Lsche Register
	movlw d'1'				;Rcksetzen des vertikalen Menpunktzhlers auf 1
	movwf M_HOR_CNT
	call wait_250_p0		;Wartezeit von 250ms Page0
	bsf PCLATH,3			;Select Page 1
	call cursor_off			;Cursor ausschalten
	call clear_display		;lsche Display
	bcf PCLATH,3			;Select Page 0
	goto main

;*******************************************************************************************
;	Unterprogramme Mensteuerung
;*******************************************************************************************

menue_up
	incf MENU_COUNT			;erhhe Menpunkt Zhler um 1
	movlw d'9'				;Maximalwert der Menpunkte +1
	subwf MENU_COUNT,w		;subtrahiere aktuellen Menpunkt nach Erhhung vom Maximalwert und prfe ob 0
	btfss STATUS, Z		
	goto menue_updn_end		;Ergebnis ist nicht 0
	movlw d'1'				;Ergebnis ist 0, Rcksetzen des Zhlers mit Wert 1
	movwf MENU_COUNT
	goto menue_updn_end
menue_dn
	decfsz MENU_COUNT		;reduziere Menpunkt Zhler um 1 und prfe ob 0
	goto menue_updn_end		;Zhler ist nicht 0
	movlw d'8'				;Zhler ist 0, Rcksetzen des Zhlers mit Maximalwert
	movwf MENU_COUNT
menue_updn_end
	call wait_250_p0		;Wartezeit von 250ms Page0
	bsf MENU_STAT,7			;setze das Statusbit
	return

inc_m_hor_cnt				;Unterprogramm zur Erhhung des Menpunktes horizontal
	movfw M_HOR_CNT_MAX		;
	subwf M_HOR_CNT,w
	btfss STATUS, Z			;prfen ob Zhler gleich dem Maximalwert ist
	goto $+4				;wenn ja, berspringe diesen Befehl	
	movlw d'1'
	movwf M_HOR_CNT
	goto inc_hor_cnt_end
	incf M_HOR_CNT
inc_hor_cnt_end
	bsf MENU_STAT,0			;setze das Status bit
	return		



inc_zalt					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt 1
	incf TEMP_11			
	movlw d'15'
	subwf TEMP_11,w
	btfss STATUS,C
	goto incdec_zalt_end
	clrf TEMP_11
	goto incdec_zalt_end
dec_zalt
	movfw TEMP_11
	btfsc STATUS,Z
	goto $+3
	decf TEMP_11
	goto incdec_zalt_end
	movlw d'14'
	movwf TEMP_11
incdec_zalt_end
	bsf PCLATH,3			;Select Page 1
	call disp_set_zalt		;
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	return

inc_zalt1
	movlw 0x00
	btfss TEMP_12,0
	movlw 0x01
	movwf TEMP_12
	bsf PCLATH,3			;Select Page 1
	call disp_set_zalt1		;
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0		;Wartezeit von 250ms Page0
	return


incdec_testrx
	btfss MENU_STAT,1
	goto $+4
	bcf MENU_STAT,1
	bsf MENU_STAT,0
	return
	bsf MENU_STAT,1
	bsf MENU_STAT,0
	return


set_fsr0_reg				;Unterprogramm zum setzen der FSR0/FSR0H Werte beim bergang von Bank1 auf Bank2 bzw. von Bank2 auf Bank3
	btfsc FSR0H,0			;prfen, ob Bank 0/1 (=>0) oder Bank 2/3 (=>1) gewhlt ist
	goto set_fsr0_b23		;Bank 2/3 (=>1) gewhlt
	movlw 0xF0				;Bank 0/1 (=>0) gewhlt, prfen, ob max. Speicherplatz +1 Bank1 (0xF0) erreicht ist
	subwf FSR0,w
	btfss STATUS,Z
	return					;nicht erreicht, Rcksprung
	bsf FSR0H,0				;max. Speicherplatz +1 (Bank 1, 0xF0) ist erreicht, setze Bit fr Bank 2/3
	movlw 0x20				;und setze FSR 0 auf die Startadresse Bank2 (0x20)
	movwf FSR0
	return					;Rcksprung
set_fsr0_b23				;Bank 2/3 (=>1) gewhlt
	movlw 0xF0				;prfen, ob max. Speicherplatz +1 Bank3 (0xF0) erreicht ist
	subwf FSR0,w
	btfss STATUS,Z
	goto $+3				;nicht erreicht, weiter
	bsf STAT_DATEN,3		;max. Speicherplatz +1 Bank3 (0xF0) ist erreicht, Setze End Bit 
	return					;Rcksprung
	movlw 0x70				;prfen, ob max. Speicherplatz +1 Bank 2 (0x70) erreicht ist 
	subwf FSR0,w
	btfss STATUS,Z
	return					;nicht erreicht, Rcksprung
	movlw 0xA0				;max. Speicherplatz +1 Bank 2 (0x70) ist erreicht, setze FSR 0 auf die Startadresse Bank3 (0xA0)
	movwf FSR0
	return					;Rcksprung

	

inc_testrx
	incf TEMP_11
	movlw d'15'
	subwf TEMP_11,w
	btfss STATUS,C
	goto incdec_testrx_end
	clrf TEMP_11
	goto incdec_testrx_end
dec_testrx
	decf TEMP_11
	movlw d'255'
	subwf TEMP_11,w
	btfss STATUS,C
	goto incdec_testrx_end
	movlw d'14'
	movwf TEMP_11
incdec_testrx_end
	bsf MENU_STAT,0
	return


inc_testpwm
	incf REGOUT_DISP
	movlw d'100'
	subwf REGOUT_DISP,w
	btfss STATUS,C
	goto incdec_testpwm_end
	movlw d'99'
	movwf REGOUT_DISP
	goto incdec_testpwm_end
dec_testpwm
	movfw REGOUT_DISP
	btfss STATUS,Z
	decf REGOUT_DISP
incdec_testpwm_end
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
							;Regout = Regout_Disp x 256 / 100
	movlw d'100'			;lade Divisionsregister Zahl_A mit Wert 100
	movwf ZAHL_A0
	movfw REGOUT_DISP		;lade Divisionsregister Zahl_C mit Regout_Disp
	movwf ZAHL_C1			;C1 => x 256
	call div32				;32 Bit Division: Zahl_C / Zahl_A = Zahl_C, Rest in Zahl_B
	call runden_div32
	movfw ZAHL_C0			;Wert an PWM Ausgang bergeben
	BANKSEL CCPR1L			;Select BANK 5
	movwf CCPR1L			; 
	BANKSEL PORTA			;Select BANK 0
	bsf MENU_STAT,0			;setze update Bit
	return

incdec_frb2
	btfss MENU_STAT,3		;Regler ein/aus
	goto $+3
	bcf MENU_STAT,3
	goto $+2
	bsf MENU_STAT,3
	bsf MENU_STAT,0			;setze update Bit
	return

befehl						;Unterprogramm zum Laden der Tasten-/IR-befehle
	clrf BEFEHL				;lsche Register fr Tasten-/IR-befehle
	btfss IN_MENUE			;prfen,ob Men Taste nicht gedrckt ist
	bsf BEFEHL,0			;Taste gedrckt, setze Bit 0
	btfss IN_OK				;prfen,ob OK Taste nicht gedrckt ist
	bsf BEFEHL,1			;Taste gedrckt, setze Bit 1
	btfss IN_RECHTS			;prfen,ob Rechts Taste nicht gedrckt ist
	bsf BEFEHL,3			;Taste gedrckt, setze Bit 3
	btfss IN_UP				;prfen,ob Up Taste nicht gedrckt ist
	bsf BEFEHL,2			;Taste gedrckt, setze Bit 2
	return

;*******************************************************************************************
;	Unterprogramme: Auswertung der Zhlerdaten und Berechnung des Reglerausgangs
;*******************************************************************************************

auswert_rx
	bcf STAT_DATEN,2		;lsche update Bit
	bcf STAT_DATEN,3		;lsche berlauf Bit
	BANKSEL TRISA			;Select BANK 1
	bcf PIE1,RCIE			;USART Receive Interrupt verbieten
	BANKSEL ANSELA			;Select BANK 3
	bcf RCSTA,CREN			;disables receiver
	BANKSEL PORTA			;Select BANK 0
	clrf FSR0H				;=>Bank 0/1
	movlw 0xA0				;FSR 0 auf start adresse setzen (0x0A0)
	movwf FSR0
	movlw 0x20				;FSR 1 auf start adresse setzen (Zhlerstand Bezug 1.8.0)
	movwf FSR1
	movwf LOOP_1
	movlw 0x28				;Maximale Adresse fr FSR 1 (es sollen die ersten 8 Byte gleich sein: 77 07 01 00 01 08 00 FF)
	movwf LOOP_2
	call startsequenz
	call set_zaehlsequenz	;setzen der Suchwerte fr Zhlerstnde 62 1E 52
	call startsequenz
	call read_zaehl_dat		;auslesen des Zhlerstands fr 1.8.0
	btfsc STAT_DATEN,3		;prfen, ob letzter Speicheradresse +1 erreicht ist (Bank 3, 0x1F0)
	goto auswert_rx_end_1
	call komma_zaehler		;umrechnen auf 2 Nachkommastellen
	movfw ZAHL_C0			;kopieren des Zhlerstands in Register 
	movwf ZAEHL_BEZUG_0
	movfw ZAHL_C1
	movwf ZAEHL_BEZUG_1
	movfw ZAHL_C2
	movwf ZAEHL_BEZUG_2
	movfw ZAHL_C3
	movwf ZAEHL_BEZUG_3

	movlw 0x20				;FSR 1 auf start adresse setzen (Zhlerstand Lieferung 2.8.0)
	movwf FSR1
	movwf LOOP_1
	movlw 0x24				;Maximale Adresse fr FSR 1 (es sollen die ersten 4 Byte gleich sein: 77 07 01 00)
	movwf LOOP_2
	call startsequenz
	movlw 0x28				;FSR 1 auf start adresse setzen
	movwf FSR1
	movwf LOOP_1
	movlw 0x2C				;Maximale Adresse fr FSR 1 (es sollen die ersten 4 Byte gleich sein: 02 08 00 FF)
	movwf LOOP_2
	call startsequenz
	call set_zaehlsequenz	;setzen der Suchwerte fr Zhlerstnde 62 1E 52
	call startsequenz
	call read_zaehl_dat		;auslesen des Zhlerstands fr 2.8.0
	btfsc STAT_DATEN,3		;prfen, ob letzter Speicheradresse +1 erreicht ist (Bank 3, 0x1F0)
	goto auswert_rx_end_1
	call komma_zaehler		;umrechnen auf 2 Nachkommastellen
	movfw ZAHL_C0			;kopieren des Zhlerstands in Register 
	movwf ZAEHL_EINSP_0
	movfw ZAHL_C1
	movwf ZAEHL_EINSP_1
	movfw ZAHL_C2
	movwf ZAEHL_EINSP_2
	movfw ZAHL_C3
	movwf ZAEHL_EINSP_3

	movlw 0x20				;FSR 1 auf start adresse setzen (Leistung Bezug/Lieferung 16.7.0)
	movwf FSR1
	movwf LOOP_1
	movlw 0x24				;Maximale Adresse fr FSR 1 (es sollen die ersten 4 Byte gleich sein: 77 07 01 00)
	movwf LOOP_2
	call startsequenz
	movlw 0x2C				;FSR 1 auf start adresse setzen
	movwf FSR1
	movwf LOOP_1
	movlw 0x30				;Maximale Adresse fr FSR 1 (es sollen die ersten 4 Byte gleich sein: 10 07 00 FF)
	movwf LOOP_2
	call startsequenz
	movlw 0x30				;setzen der Suchwerte fr Leistung 62 1B 52
	movwf FSR1
	movlw 0x62
	movwi FSR1++
	movlw 0x1B
	movwi FSR1++
	movlw 0x52
	movwf INDF1
	movlw 0x30				;FSR 1 auf start adresse setzen
	movwf FSR1
	movwf LOOP_1
	movlw 0x33				;Maximale Adresse fr FSR 1
	movwf LOOP_2
	call startsequenz
	call read_zaehl_dat
	btfsc STAT_DATEN,3		;prfen, ob letzter Speicheradresse +1 erreicht ist (Bank 3, 0x1F0)
	goto auswert_rx_end_1
	btfsc STAT_DATEN,5
	call p_negativ
	call komma_leistung
	movfw ZAHL_C0			;kopiere Zahl C register nach P_inout Register
	movwf P_INOUT_L
	movfw ZAHL_C1
	movwf P_INOUT_M
	movfw ZAHL_C2
	movwf P_INOUT_H
	btfsc STAT_DATEN,5
	bsf P_INOUT_H,7			;setze Bit fr Minus

	btfsc STAT_PWM,0		;prfen, ob Bit fr "Regler Aus" nicht gesetzt ist
	goto auswert_rx_end		;Bit fr "Regler Aus" ist gesetzt, weiter bei auswert_rx_end

regelabweichung				;Regelabweichung berechnen
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0
	movfw P_INOUT_H			;Regelabweichung = P_INOUT x 256 / P_LAST 
	movwf ZAHL_C3			;lade Divisionsregister Zahl C mit der aktuellen Leistung P_INOUT ( 8 Bit nach links => x256 )
	bcf ZAHL_C3,7			;lsche Bit 7 (Vorzeichen)
	movfw P_INOUT_M
	movwf ZAHL_C2
	movfw P_INOUT_L
	movwf ZAHL_C1
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	movfw P_LAST_L			;lade Divisionsregister Zahl A mit der max. Leistung P_LAST 
	movwf ZAHL_A0
	movfw P_LAST_H
	movwf ZAHL_A1
	call div32				;32 Bit Division: ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B
	call runden_div32		;Ganzzahliges runden
	movfw ZAHL_C0			;Kopiere das L-Byte des Ergebnis ins REGELABW Register
	movwf REGELABW
	movfw ZAHL_C2			;prfen, ob REGELABW <256, d.h. P_INOUT < P_LAST, (wenn Zahl C1 + C2 = 0, P_INOUT < P_LAST)
	btfss STATUS,Z			;prfen ob Zahl C2 =0
	goto $+4				;Zahl C2 nicht 0, schreibe Wert 255 ins REGELABW Register
	movfw ZAHL_C1			;Zahl C2 =0, prfen ob Zahl C1 =0
	btfsc STATUS,Z
	goto $+3				;Zahl C1 =0, weiter
	movlw 0xFF				;Zahl C1 nicht 0, schreibe Wert 255 ins REGELABW Register
	movwf REGELABW

;P-Anteil berechnen
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0
	movfw REGLER_KP			;Kp umrechnen: Kp(x100) x 256 / 100, Kp(x100) in Divisionsregister ZAHL_C kopieren => (Zahl C1 => x256)
	movwf ZAHL_C1
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	movlw 0x64				;Wert 100 in 2tes Divisionsregister ZAHL_A kopieren
	movwf ZAHL_A0
	call div32				;Division ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B, Kp (0-255) steht in ZAHL_C
	call runden_div32		;Ganzzahliges runden 
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	movfw ZAHL_C0			;P-Anteil (0-255) = Kp (0-255) x Regelabweichung (0-255) / 256, kopiere Zahl C0 nach Zahl A0
	movwf ZAHL_A0
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movfw REGELABW			;kopiere REGELABW nach Zahl B0
	movwf ZAHL_B0
	call mul32				;32 Bit Multiplikation: ZAHL_A x ZAHL_B = ZAHL_C
	movfw ZAHL_C1			;kopiere Zahl C1 nach P-Anteil ( C1 => / 256)
	movwf P_ANTEIL

auswert_rx_end
	btfss STAT_PWM,7		;prfen, ob am Ausgang Pin8 (RB2) die aktuelle Leistung Pb bzw. Pe ausgegeben werden soll
	goto abschalt_timer		;nein, Ausgang Pin8 (RB2) ist als on/off Pin fr Abschalttimer gewhlt

	movfw P_INOUT_H			;Leistung Bezug/Einspeisung (16.7.0), H Byte
	BANKSEL ANSELA			;Select BANK 3
	btfss TXSTA,TRMT		;prfen, ob TSR Register leer
	goto $-1				;nicht leer, erneut prfen
	movwf TXREG				;Register ist leer, Byte kann transferriert werden
	BANKSEL PORTA			;Select BANK 0
	movfw P_INOUT_M			;Leistung Bezug/Einspeisung (16.7.0), M Byte
	BANKSEL ANSELA			;Select BANK 3
	btfss TXSTA,TRMT		;prfen, ob TSR Register leer
	goto $-1				;nicht leer, erneut prfen
	movwf TXREG				;Register ist leer, Byte kann transferriert werden
	BANKSEL PORTA			;Select BANK 0
	movfw P_INOUT_L			;Leistung Bezug/Einspeisung (16.7.0), L Byte
	BANKSEL ANSELA			;Select BANK 3
	btfss TXSTA,TRMT		;prfen, ob TSR Register leer
	goto $-1				;nicht leer, erneut prfen
	movwf TXREG				;Register ist leer, Byte kann transferriert werden
	BANKSEL PORTA			;Select BANK 0
	goto auswert_rx_end_1

abschalt_timer
	btfss P_INOUT_H,7		;prfen, ob Bit fr minus gesetzt ist (Einspeisung)
	goto auswert_rx_end_1	;Bit nicht gesetzt, weiter 
	clrf TIMER_ABSCH		;lsche Timer
	btfsc OUT_ON_OFF		;prfen, ob Ausgang nicht gesetzt ist
	goto auswert_rx_end_1	;Ausgang gesetzt, weiter
	btfss STAT_PWM,6		;Ausgang nicht gesetzt, prfen, ob das zweite Mal in Folge eine Einspeisung detektiert wurde
	goto $+3				;bit nicht mehr gesetzt, d.h. das zweite Mal in Folge Einspeisung detektiert
	bcf STAT_PWM,6			;bit noch gesetzt, das erste Mal Einspeisung detektiert, lsche Statusbit
	goto auswert_rx_end_1
	bsf OUT_ON_OFF			;setze Ausgang

auswert_rx_end_1
	call clear_reg			;lsche Register Bank 1,2,3 und rcksetzen der FSR 0 und FSR 0H Werte
	movlw 0x20				;FSR 1 auf start adresse setzen
	movwf FSR1
	movwf FSR_REG_1
	BANKSEL ANSELA			;Select BANK 3
	bsf RCSTA,CREN			;Enables receiver
	BANKSEL TRISA			;Select BANK 1
	bsf PIE1,RCIE			;USART Receive Interrupt erlauben
	BANKSEL PORTA			;Select BANK 0
	return

startsequenz
	call set_fsr0_reg		;setzen der FSR 0 Adressen beim bergang von Bank1 auf Bank2 bzw. von Bank2 auf Bank3
	btfsc STAT_DATEN,3		;prfen, ob letzter Speicheradresse +1 erreicht ist (Bank 3, 0x1F0)
	return
	movfw INDF1				;prfen, ob Wert in FSR 0 dem gesuchten Wert in FSR 1 entspricht
	subwf INDF0,w
	btfsc STATUS,Z
	goto $+5				;ja, Werte sind gleich, weiter bei $+5
	movfw LOOP_1			;nein Werte sind nicht gleich, zurcksetzen von FSR 1 und erhhen von FSR 0
	movwf FSR1
	incf FSR0
	goto startsequenz		;zurck zur Schleife
	incf FSR0				;ja, Werte sind gleich, erhhe FSR 0 und FSR 1
	incf FSR1
	movfw LOOP_2			;prfen, ob FSR 1 den Maximalwert berschritten hat
	subwf FSR1,w
	btfss STATUS,Z
	goto startsequenz		;nein, Maximalwert noch nicht berschritten, zurck zur Schleife
	return					;ja, Maximalwert ist berschritten, Rcksprung

set_zaehlsequenz
	movlw 0x30				;setzen der Suchwerte fr Zhlerstnde 62 1E 52
	movwf FSR1
	movlw 0x62				;hole 0x62 nach W
	movwi FSR1++			;schiebe W nach INDF1 und erhhe FSR1 um 1
	movlw 0x1E				;hole 0x1E nach W
	movwi FSR1++			;schiebe W nach INDF1 und erhhe FSR1 um 1
	movlw 0x52				;hole 0x52 nach W
	movwf INDF1				;schiebe W nach INDF1
	movlw 0x30				;FSR 1 auf start adresse setzen
	movwf FSR1
	movwf LOOP_1
	movlw 0x33				;Maximale Adresse fr FSR 1
	movwf LOOP_2
	return

read_zaehl_dat
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0	
	call set_fsr0_reg		;setzen der FSR 0 Adressen beim bergang von Bank1 auf Bank2 bzw. von Bank2 auf Bank3
	btfsc STAT_DATEN,3		;prfen, ob letzter Speicheradresse +1 erreicht ist (Bank 3, 0x1F0)
	return
	incf FSR0				;berspringe 1 Byte (wird nicht bentigt)
	call set_fsr0_reg		;setzen der FSR 0 Adressen beim bergang von Bank1 auf Bank2 bzw. von Bank2 auf Bank3
	btfsc STAT_DATEN,3		;prfen, ob letzter Speicheradresse +1 erreicht ist (Bank 3, 0x1F0)
	return
	moviw FSR0++
	call laengen_byte	
	addlw 0x76
	movwf FSR1

	bcf STAT_DATEN,5		;lsche Statusbit fr negative Leistung (Einspeisung)
	btfsc INDF0,7			;prfen, ob das hchste Byte <128 (bei Leistung positiv, d.h. Bezug)
	bsf STAT_DATEN,5		;>=128, setze Statusbit fr negative Leistung (Einspeisung)

	clrf FSR1H				;FSR 1H auf Bank 0/1 setzen
	call set_fsr0_reg		;setzen der FSR 0 Adressen beim bergang von Bank1 auf Bank2 bzw. von Bank2 auf Bank3
	btfsc STAT_DATEN,3		;prfen, ob letzter Speicheradresse +1 erreicht ist (Bank 3, 0x1F0)
	return
	moviw FSR0++
	movwi FSR1-- 
	movlw 0x77
	subwf FSR1,w
	btfss STATUS,Z
	goto $-8
	movlw 0x02				;FSR 1H wieder auf Bank 4/5 setzen
	movwf FSR1H
	return


laengen_byte				;Unterprogrammm zum Auftrennen eines 8 bit Zeichens in die 2 Zeichen des hex Formats: XY
							;X = Auflsung: 4 = 10W(bzw.Wh), 5 = 1W(bzw.Wh), 6 = 0,1W(bzw.Wh) ; Y = Anzahl Bytes-1
	movwf TEMP_13			;kopiere W in Temp register, Temp 13 = Anzahl Bytes minus 1
	movwf TEMP_14			;Temp 14 = Nachkommastellen: 5 = 0 bei W bzw. 3 bei kW, 6 = 1 bei W bzw. 4 bei kW
	bcf TEMP_13,7			;lsche obere 4 Bit
	bcf TEMP_13,6
	bcf TEMP_13,5
	bcf TEMP_13,4	
	movlw 0x0A				;prfen, ob Y im Bereich der gltigen Anzahl Bytes liegt (min = 1, max = 8 ; Ymin =2, Ymax = 9)
	subwf TEMP_13,w			;Subtraktion temp13 - 10
	btfss STATUS,C			;prfen, ob Ergebnis positiv, d.h. Y >9, => zu gro
	goto $+4				;Y <=9, ok, weiter bei Prfung auf Minimum
	movlw 0x09				;Y >9, Begrenzen auf Wert 9
	movwf TEMP_13
	goto trennen_hex1
	movlw 0x02				;prfen, ob Y im Bereich der gltigen Anzahl Bytes liegt (min = 1, max = 8 ; Ymin =2, Ymax = 9)
	subwf TEMP_13,w			;Subtraktion temp13 - 2
	btfsc STATUS,C			;prfen, ob Ergebnis negativ, d.h. Y <2, => zu klein
	goto trennen_hex1		;Y >=2, ok, weiter
	movlw 0x02				;Y <2, Begrenzen auf Wert 2
	movwf TEMP_13
trennen_hex1
	swapf TEMP_14			;vertausche obere und untere 4 Bit, Lngenbyte: Temp 14 = Nachkommastellen
	bcf TEMP_14,7			;lsche obere 4 Bit
	bcf TEMP_14,6
	bcf TEMP_14,5
	bcf TEMP_14,4
	movfw TEMP_13			;Lngenbyte: Temp 13: Anzahl Bytes minus 1
	return

komma_zaehler
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	movlw 0x05				;hole Wert 5 nach W (Auflsung 1W)
	subwf TEMP_14,w
	btfss STATUS,Z			;prfen, ob 0
	goto $+5				;>0<, weiter bei nchster Prfung
	movlw 0x0A				;=5, lade Divisionsregister Zahl_A mit Wert 10
	movwf ZAHL_A0
	call div32				;Division Zahl_C / Zahl_A
	return
	movlw 0x06				;hole Wert 6 nach W (Auflsung 0,1W)
	subwf TEMP_14,w
	btfss STATUS,Z			;prfen, ob 0
	return					;>0<, d.h. Wert ist 4 (keine Anpassung erforderlich) oder >6 bzw. <4 (dann Fehler, wird aber ignoriert)
	movlw 0x64				;=6, lade Divisionsregister Zahl_A mit Wert 100
	movwf ZAHL_A0
	call div32				;Division Zahl_C / Zahl_A
	return

komma_leistung
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	movlw 0x0A				;lade register Zahl_A mit Wert 10 fr Multiplikation oder Division
	movwf ZAHL_A0

	movlw 0x06				;hole Wert 6 nach W (Auflsung 0,1W)
	subwf TEMP_14,w
	btfss STATUS,Z			;prfen, ob 0
	goto $+3				;>0<, weiter bei nchster Prfung
	call div32				;Division Zahl_C / Zahl_A
	return

	movlw 0x04				;hole Wert 4 nach W (Auflsung 10W)
	subwf TEMP_14,w
	btfss STATUS,Z			;prfen, ob 0
	return					;>0<, d.h. Wert ist 5 (keine Anpassung erforderlich) oder >6 bzw. <4 (dann Fehler, wird aber ignoriert)
	movfw ZAHL_C0
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	movfw ZAHL_C2
	movwf ZAHL_B2
	movfw ZAHL_C3
	movwf ZAHL_B3
	call mul32				;multiplikation Zahl_A x Zahl_B = Zahl_C
	return

p_negativ					;kopiere die 4 unteren Byte von Zahl C nach Zahl B fr die Subtraktion. Wenn die Anzahl
							;der bertragenen Bytes <4, ersetze die fehlenden Bytes durch 0xFF
	movlw 0x05				;hole wert 5 nach W 
	subwf TEMP_13,w			;subtrahiere W von Anzahl Byte-1
	movfw ZAHL_C3			;hole Zahl C3 Register nach W
	btfss STATUS,C			;prfen, ob Ergebnis der Subtraktion positiv
	movlw 0xFF				;Ergebnis negativ, kopiere Wert 0xFF nach Zahl B3 Register
	movwf ZAHL_B3			;Ergebnis positiv, kopiere Zahl C3 Register nach Zahl B3 Register

	movlw 0x04				;hole wert 4 nach W 
	subwf TEMP_13,w			;subtrahiere W von Anzahl Byte-1
	movfw ZAHL_C2			;hole Zahl C2 Register nach W
	btfss STATUS,C			;prfen, ob Ergebnis der Subtraktion positiv
	movlw 0xFF				;Ergebnis negativ, kopiere Wert 0xFF nach Zahl B2 Register
	movwf ZAHL_B2			;Ergebnis positiv, kopiere Zahl C2 Register nach Zahl B2 Register

	movlw 0x03				;hole wert 3 nach W 
	subwf TEMP_13,w			;subtrahiere W von Anzahl Byte-1
	movfw ZAHL_C1			;hole Zahl C1 Register nach W
	btfss STATUS,C			;prfen, ob Ergebnis der Subtraktion positiv
	movlw 0xFF				;Ergebnis negativ, kopiere Wert 0xFF nach Zahl B1 Register
	movwf ZAHL_B1			;Ergebnis positiv, kopiere Zahl C1 Register nach Zahl B1 Register

	movfw ZAHL_C0			;
	movwf ZAHL_B0
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0	
	call sub32
	clrf ZAHL_C4
	return

produktion					;Unterprogramm zur Berechnung der PV leistung aus der Zeit zwischen 2 Impulsen
	bcf STAT_DATEN,0
	btfss STAT_DATEN,1		;
	goto $+4
	clrf P_PROD_H
	clrf P_PROD_L
	return
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0	
	movlw 0x68				;lade Zahl 36.000.000.000 (0x0861C46800)
	movwf ZAHL_C1
	movlw 0xC4
	movwf ZAHL_C2
	movlw 0x61
	movwf ZAHL_C3
	movlw 0x08
	movwf ZAHL_C4
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0	
	movfw ZPROD_IMP			;kopiere Wert fr Imp/kWh nach Zahl A0
	movwf ZAHL_A0
	call div32				;32 bit Division
	movfw T_PROD_0
	movwf ZAHL_A0
	movfw T_PROD_1
	movwf ZAHL_A1
	movfw T_PROD_2
	movwf ZAHL_A2
	movfw T_PROD_3
	movwf ZAHL_A3
	call div32				;32 bit Division
	movfw ZAHL_C0
	movwf P_PROD_L
	movfw ZAHL_C1
	movwf P_PROD_H
	return

clear_reg					;Unterprogramm zum lschen der Register 0x0A0 bis 0x0EF (Bank2) und 0x120 bis 0x1EF (Bank3/4)
	clrf FSR0H				;lsche IRP Bit (indirekte Adressierung Bank0/1)
	movlw 0xA0				;lade Erste Adresse in FSR0 Register
	movwf FSR0
	movlw 0xF0				;maximale adresse +1
	movwf LOOP_1
	call clear_reg_loop

	bsf FSR0H,0				;setze IRP Bit (indirekte Adressierung Bank2/3)
	movlw 0x20				;lade Erste Adresse in FSR0 Register
	movwf FSR0
	movlw 0x70				;maximale adresse +1
	movwf LOOP_1
	call clear_reg_loop

	movlw 0xA0				;lade Erste Adresse in FSR0 Register
	movwf FSR0
	movlw 0xF0				;maximale adresse +1
	movwf LOOP_1
	call clear_reg_loop

	clrf FSR0H				;lsche IRP Bit (indirekte Adressierung Bank0/1)
	clrf FSR_REG_0H
	movlw 0xA0				;rcksetzen des FSR0 Register
	movwf FSR0
	movwf FSR_REG_0
	movlw 0x20				;rcksetzen des FSR0 Register
	movwf FSR1
	movwf FSR_REG_1
	return					;fertig

clear_reg_loop
	clrf INDF0				;lsche INDF0 Register
	incf FSR0,F 			;Erhhe FSR0 Register
	movfw LOOP_1			;maximale adresse +1
	subwf FSR0,w			;subtrahiere FSR0 in w
	btfss STATUS,Z			;prfen, ob 0, d.h. maximale adresse +1 erreicht ist
	goto clear_reg_loop		;maximale adresse +1 noch nicht erreicht, zurck zur Schleife
	return



disp_inc_man
	btfsc STAT_DATEN,4
	goto $+3
	bsf STAT_DATEN,4
	goto disp_incdec_end
	incf DISP_NR
	movlw 0x04
	subwf DISP_NR,w
	btfsc STATUS,C
	clrf DISP_NR
	goto disp_incdec_end
disp_dec_man
	btfsc STAT_DATEN,4
	goto $+3
	bsf STAT_DATEN,4
	goto disp_incdec_end
	movlw 0x01
	subwf DISP_NR
	btfsc STATUS,C
	goto disp_incdec_end
	movlw 0x03
	movwf DISP_NR
disp_incdec_end
	bsf PCLATH,3			;Select Page 1
	call disp_main
	bcf PCLATH,3			;Select Page 0
	call wait_250_p0
	return


disp_inc_auto
	bcf STAT_TIMER,3
	incf DISP_NR
	movlw 0x03
	subwf DISP_NR,w
	btfsc STATUS,C
	clrf DISP_NR
	return	

test_p_prod
	bcf STAT_TIMER,4
	btfss STAT_DATEN,1		;
	goto $+4
	clrf P_PROD_H
	clrf P_PROD_L
	return

	movfw TMR1_EXT2
	movwf ZAHL_B3
	movfw TMR1_EXT1
	movwf ZAHL_B2
	movfw TMR1H
	movwf ZAHL_B1
	movfw TMR1
	movwf ZAHL_B0
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0	
	movfw T_PROD_3
	movwf ZAHL_C3
	movfw T_PROD_2
	movwf ZAHL_C2
	movfw T_PROD_1
	movwf ZAHL_C1
	movfw T_PROD_0
	movwf ZAHL_C0
	call sub32				;Subtraktion Zahl C - Zahl B, Talt -Tneu 
	btfss ZAHL_C4,7			;prfen, ob Ergebnis negativ
	return					;Ergebnis positiv, zurck
	movfw TMR1
	movwf T_PROD_0
	movfw TMR1H
	movwf T_PROD_1
	movfw TMR1_EXT1
	movwf T_PROD_2
	movfw TMR1_EXT2
	movwf T_PROD_3
	goto produktion			;Ergebnis negativ, neue, kleinere Leistung berechnen


store_z_vt					;Unterprogramm zum Speichern der Zhlerstnde des Vortages
	bcf STAT_TIMER,5		;lsche Statusbit fr neuen Tag
	movfw EEPROM_ADR		;EEPROM Adresse der zuletzt gespeicherten Zhlerstnde
	addlw d'12'
	movwf EEPROM_ADR
	sublw d'184'
	btfsc STATUS,C
	goto $+3
	movlw d'28'
	movwf EEPROM_ADR
	movfw EEPROM_ADR
	movwf TEMP_13
	movlw 0x00				;EEPROM Adresse der zuletzt gespeicherten Zhlerstnde
	bsf PCLATH,3			;Select Page 1
	call writeEE			;Schreiben in das interne EEPROM
	bcf PCLATH,3			;Select Page 0
	movfw EEPROM_ADR
	call store_zaehler
	movlw 0x01
	subwf TAG,w
	btfss STATUS,Z
	return
	movlw d'16'
	call store_zaehler	
	return


store_zaehler
	movwf TEMP_11			;schiebe W in Temp Register: EEPROM Adressregister
	clrf FSR0H				;lsche IRP Bit (indirekte Adressierung Bank0/1)
	movlw 0x46				;Speicherplatz Zhlerstand Bezug (1.8.0), Byte3
	movwf FSR0
store_zaeh_loop
	moviw FSR0++			;hole Wert nach W und erhhe FSR0 um 1
	movwf TEMP_13			;EEPROM Schreibregister
	movfw TEMP_11			;EEPROM Adressregister
	bsf PCLATH,3			;Select Page 1
	call writeEE			;Schreiben in das interne EEPROM
	bcf PCLATH,3			;Select Page 0
	incf TEMP_11			;Erhhe EEPROM Adressregister
	movlw 0x52				;Speicherplatz (+1) Zhlerstand Prodduktion, Byte0	
	subwf FSR0,w			;subtrahiere FSR0 in W
	btfss STATUS,Z			;prfen ob 0, d.h. max. Speicherplatz +1 schon erreicht
	goto store_zaeh_loop	;noch nicht erreicht, zurck zur Schleife
	bsf FSR0H,0				;setze IRP Bit (indirekte Adressierung Bank2/3)
	movlw 0x20				;rcksetzen von FSR0 auf 0x20
	movwf FSR0
	return


wait_250_p0					;Unterprogramm fr Wartezeit 250ms Page0
	movlw d'250'
wait_p0						;Unterprogramm fr Wartezeit Xms Page0
	movwf LOOP_1
loop_250_p0
	movlw d'250'			;loop_10 mu 4 Takten entsprechen fr 1ms: 4MHz /4/250/4Takte
	movwf LOOP_2
loop_10_p0
	nop
	decfsz LOOP_2
	goto loop_10_p0
	decfsz LOOP_1
	goto loop_250_p0
	return



update_reg_i
	bcf STAT_TIMER,6	;lsche Timer Bit fr 200 ms
;temp_messen
	bcf STAT_PWM,1		;lsche Bits fr Fhlerbruch/Kurzschluss
	bcf STAT_PWM,2
	BANKSEL TRISA		;Select BANK 1	
	bsf	ADCON0,1		; ADC starten
	btfsc ADCON0,1		; prfen ob der ADC fertig ist?
	goto $-1			; nein, weiter warten

	movlw d'21'			;minimaler AD Wert
	subwf ADRESH,w		;subtrahiere minimalen AD Wert von den oberen 8 Bit des AD Wertes
	BANKSEL PORTA		;Select BANK 0
	btfss STATUS,C		;prfen ob Ergebnis positiv, d.h. AD Wert >= min
	bsf STAT_PWM,2		;Ergebnis negativ, d.h. AD Wert < min, setze Fehlerbit fr "Fhler Kurzschluss"
	BANKSEL TRISA		;Select BANK 1	
	movlw d'236'		;maximaler AD Wert plus 1
	subwf ADRESH,w		;subtrahiere minimalen AD Wert von den oberen 8 Bit des AD Wertes
	BANKSEL PORTA		;Select BANK 0
	btfsc STATUS,C		;prfen ob Ergebnis negativ, d.h. AD Wert < max +1
	bsf STAT_PWM,1		;Ergebnis positiv, d.h. AD Wert >= max +1, setze Fehlerbit fr "Fhlerbruch"
	BANKSEL TRISA		;Select BANK 1	
	movfw ADRESH		;AD Wert im gltigen Bereich, obere 8 Bit auslesen
	BANKSEL PORTA		;Select BANK 0
	movwf T_H			;kopiere den AD Wert (8 Bit) fr die Heizungstemperatur ins entsprechende Register
	btfsc STAT_PWM,1	;prfen, ob Fehlerbit fr "Fhlerbruch Heizung" gesetzt ist
	goto pwm_off		;Heizung ausschalten
	btfsc STAT_PWM,2	;prfen, ob Fehlerbit fr "Kurzschluss Fhler Heizung" gesetzt ist
	goto pwm_off		;ja, Heizung ausschalten
	movlw 0x0F			;setzen des Programmzhlers fr Temperaturtabelle im Bereich 0x0F00...0x0FFF
	movwf PCLATH
	movlw d'21'
	subwf T_H,w			;hole den AD Wert (8 Bit) fr die Heizungstemperatur nach W
	bsf PCLATH,3		;Select Page 1
	call tab_temperatur	;hole den entsprechenden Temperaturwert aus der Temperaturtabelle
	bcf PCLATH,3		;Select Page 0
	movwf T_H			;(Umrechnung der Tabellenwerte in Temperatur: Tabellenwert /2 minus 10)
						;wenn Heizungstemperatur 4C unter Maximaler Temperatur erreicht ist, beginnt eine Leistungsbeschrenkung
	movfw T_MAX				;Subtrahiere Temperatur Heizung von maximaler Temperatur in w (Basis: Tabellenwerte)
	subwf T_H,w
	btfss STATUS,C			;prfen, ob T_H < T_MAX, 
	goto $+7				;T_H < T_MAX, weiter bei nchster Prfung
	clrf REGOUT				;T_H >= T_MAX, setze Regelausgang auf 0
	clrf I_ANTEIL_L			;I-Anteil auf Null setzen
	clrf I_ANTEIL_M
	clrf I_ANTEIL_H
	bsf STAT_PWM,4			;setze Bit fr bertemperatur
	goto add_p_anteil_end

	bcf STAT_PWM,4			;lsche Bit fr bertemperatur
	movfw T_H				;Subtrahiere Temperatur Heizung +4C (=>Tabellenwerte 8) von maximaler Temperatur in w (Basis: Tabellenwerte)
	addlw 0x08
	subwf T_MAX,w
	btfsc STATUS,C			;prfen, ob T_H +8 > T_MAX, 
	goto update_i_anteil	;T_H +8 </= T_MAX, keine Leistungsreduzierung

	movfw T_H				;Subtrahiere Temperatur Heizung von maximaler Temperatur in w (Basis: Tabellenwerte)
	subwf T_MAX,w
	movwf LOOP_2			;speichern der Differenz in Loop 2, pro 0,5C wird der Regelausgang um 1/8tel bzw. 12,5% reduziert
	clrf LOOP_1				;lsche Register
	movlw 0x20				;hole Wert 32 nach W (256 / 8 = 32)
	addwf LOOP_1			;
	decfsz LOOP_2
	goto $-3
	bsf STAT_PWM,5			;setze Bit fr Reglerbegrenzung, der Begrenzungswert steht in Loop1

update_i_anteil 
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	movlw 0x33				;=> 0,2s x 65536 x Regelabweichung / Tn => 13107 (0x3333) x Regelabweichung / Tn
	movwf ZAHL_A0			;lade ZAHL_A Register mit Wert 13107 (0x3333)
	movlw 0x33
	movwf ZAHL_A1
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movfw REGELABW			;lade ZAHL_B Register mit Regelabweichung
	movwf ZAHL_B0
	call mul32				;32 Bit Multiplikation: ZAHL_A x ZAHL_B = ZAHL_C
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	movfw REGLER_TN			;lade ZAHL_A Register mit Tn
	movwf ZAHL_A0
	call div32				;32 Bit Division: ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B
	call runden_div32		;ganzzahliges runden 

	movfw ZAHL_C0
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	movfw ZAHL_C2
	movwf ZAHL_B2
	movfw ZAHL_C3
	movwf ZAHL_B3
	call clear_zahlc_p0		;Unterprogramm zum Lschen der ZAHL_C Register, Page 0
	movfw I_ANTEIL_L
	movwf ZAHL_C0
	movfw I_ANTEIL_M
	movwf ZAHL_C1
	movfw I_ANTEIL_H
	movwf ZAHL_C2
	btfss P_INOUT_H,7		;prfen, ob Bezug (0) oder Einspeisung (1)
	goto sub_i_anteil
	call add32
	movfw ZAHL_C3			;prfen, ob I-Anteil bergelaufen ist
	btfsc STATUS,Z	
	goto update_i_end		;nein, nicht bergelaufen
	movlw 0xFF				;ja, bergelaufen, begrenze I-Anteil auf 0xFFFFFF
	movwf I_ANTEIL_L
	movwf I_ANTEIL_M
	movwf I_ANTEIL_H
	goto add_p_anteil
	
sub_i_anteil
	call sub32
	movfw ZAHL_C3			;prfe I-Anteil auf Unterlauf
	btfsc STATUS,Z	
	goto update_i_end		;kein Unterlauf
	movlw 0x00				;Unterlauf, begrenze I-Anteil auf 0x000000
	movwf I_ANTEIL_L
	movwf I_ANTEIL_M
	movwf I_ANTEIL_H
	goto add_p_anteil
	
update_i_end
	movfw ZAHL_C0
	movwf I_ANTEIL_L
	movfw ZAHL_C1
	movwf I_ANTEIL_M
	movfw ZAHL_C2
	movwf I_ANTEIL_H

add_p_anteil				;P + I Anteil addieren
	movfw P_ANTEIL
	btfss P_INOUT_H,7		;prfen, ob Bezug (0) oder Einspeisung (1)
	goto sub_p_anteil		;Bezug, subtrahiere P-Anteil von I-Anteil
	addwf I_ANTEIL_H,w		;Einspeisung, addiere P-Anteil und I-Anteil (H-Byte)
	btfss STATUS,C			;prfen auf berlauf
	goto $+4				;Kein berlauf
	movlw 0xFF				;berlauf, begrenze Reglerausgangsregister auf 0xFF (255)
	movwf REGOUT			;
	goto test_begrenzung
	movwf REGOUT			;Kein berlauf, kopiere Additionsergebnis in Reglerausgangsregister
	goto test_begrenzung
sub_p_anteil
	subwf I_ANTEIL_H,w
	btfsc STATUS,C			;prfen auf Unterlauf
	goto $+4				;Kein Unterlauf
	movlw 0x00				;Unterlauf, begrenze Reglerausgangsregister auf 0x00 (0)
	movwf REGOUT			;
	goto test_begrenzung
	movwf REGOUT			;Kein berlauf, kopiere Additionsergebnis in Reglerausgangsregister

test_begrenzung
	btfss STAT_PWM,5		;prfen, ob der Regelausgang begrenzt werden soll
	goto add_p_anteil_end	;nein, weiter
	bcf STAT_PWM,5			;Lsche Statusbit
	movfw REGOUT			;Loop1 minus Regout, Loop1 ist Wert fr Begrenzung
	subwf LOOP_1,w
	btfsc STATUS,C			;prfen ob Ergebniss negativ, d.h. Regout > Begrenzung
	goto add_p_anteil_end	;Ergebniss positiv, weiter
	movfw LOOP_1			;Ergebniss negativ, Begrenze Regout
	movwf REGOUT
	clrf I_ANTEIL_L			;lsche L+M Register
	clrf I_ANTEIL_M
	movfw P_ANTEIL
	btfss P_INOUT_H,7		;prfen, ob Bezug (0) oder Einspeisung (1)
	goto $+3				;Bezug, addiere P-Anteil und Regout
	subwf REGOUT,w			;Einspeisung, subtrahiere P-Anteil von Regout
	goto $+2
	addwf REGOUT,w			;Bezug, addiere P-Anteil und Regout
	movwf I_ANTEIL_H		;

add_p_anteil_end			;
	call clear_zahla_p0		;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movfw REGOUT			;
	BANKSEL CCPR1L			;Select BANK 5
	movwf CCPR1L			; 
	BANKSEL PORTA			;Select BANK 0
	movwf ZAHL_A0
	movlw d'100'
	movwf ZAHL_B0
	call mul32
	btfsc ZAHL_C0,7
	incf ZAHL_C1
	movfw ZAHL_C1
	movwf REGOUT_DISP
	return


pwm_off
	BANKSEL CCPR1L			;Select BANK 5
	clrf CCPR1L				;Lsche PWM Ausgaberegister
	BANKSEL PORTA			;Select BANK 0
	clrf REGOUT				;Lsche Register
	clrf REGOUT_DISP
	clrf P_ANTEIL
	clrf I_ANTEIL_L
	clrf I_ANTEIL_M
	clrf I_ANTEIL_H
	return


;*******************************************************************************************
;	Unterprogramme Division, Multiplikation, Addition und Subtraktion
;*******************************************************************************************

;*******************************************************************************************
; 32 bit Division: ZAHL_C / ZAHL_A = ZAHL_C, Rest steht in ZAHL_B
; Die Werte fr den Zhler mssen in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H) stehen
; Die Werte fr den Nenner mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) stehen
; Das ganzzahlige Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
; Der Rest steht in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
;*******************************************************************************************

div32
	call clear_zahlb_p0		;Rest von vorherigen Berechnungen lschen, ZAHL_B
	movlw d'64'				;LOOP_2 mit 64 laden, da 64 bit im Zhler
	movwf LOOP_2
div32_loop
	rlf ZAHL_C0,F       	;den gesamtem Zhler um 1 nach links schieben, ZAHL_C
	rlf ZAHL_C1,F
	rlf ZAHL_C2,F
	rlf ZAHL_C3,F
	rlf ZAHL_C4,F
	rlf ZAHL_C5,F
	rlf ZAHL_C6,F
	rlf ZAHL_C7,F

	rlf ZAHL_B0,F			;das Register fr den Rest um 1 nach links schieben
	rlf ZAHL_B1,F
	rlf ZAHL_B2,F
	rlf ZAHL_B3,F

	rlf LOOP_2,F			;das Carrybit wird vorbergehend im LOOP_2 gespeichert
									
	bsf LOOP_2,0
	movf ZAHL_A0,W			;Subtrahiere Nenner _0 von Rest _0 (REST_0-NENNER_0=REST_0)
	subwf ZAHL_B0,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B1,F			;reduziere REST_1 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_2,0			;bertrag vorhanden, lsche Bit 0 in LOOP_2

div32_sub2
	movf ZAHL_A1,W			;Subtrahiere Nenner _1 von Rest _1 (REST_1-NENNER_1=REST_1)
	subwf ZAHL_B1,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_2,0			;bertrag vorhanden, lsche Bit 0 in LOOP_2

div32_sub3
	movf ZAHL_A2,W			;Subtrahiere Nenner _2 von Rest _2 (REST_2-NENNER_2=REST_2)
	subwf ZAHL_B2,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub4			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_2,0			;bertrag vorhanden, lsche Bit 0 in LOOP_2

div32_sub4
	movf ZAHL_A3,W			;Subtrahiere Nenner _3 von Rest _3 (REST_3-NENNER_3=REST_3)
	subwf ZAHL_B3,F			;

	btfss STATUS, C			;prfe, ob das Ergebnis positiv ist
	bcf LOOP_2,0			;wenn nein, lsche gespeichertes Carrybit in LOOP_2
	btfsc LOOP_2,0			;prfe das gespeicherte Carrybit in LOOP_2
	goto div32_jump			;wenn Carry=1: springe nach div16_jump

	movf ZAHL_A0,W			;wenn Carry=0:	Subtraktion wieder rckgngig machen 
	addwf ZAHL_B0,F			; 
	btfsc STATUS,C
	incfsz ZAHL_B1,F
	goto $+4
	incfsz ZAHL_B2,F
	goto $+2
	incf ZAHL_B3,F

	movf ZAHL_A1,W			;
	addwf ZAHL_B1,F			;
	btfsc STATUS,C
	incfsz ZAHL_B2,F
	goto $+2
	incf ZAHL_B3,F

	movf ZAHL_A2,W			; 
	addwf ZAHL_B2,F			; 
	btfsc STATUS,C
	incf ZAHL_B3,F

	movf ZAHL_A3,W			;
	addwf ZAHL_B3,F			; 

div32_jump
	bcf STATUS,C			;Carrybit lschen und das gespeicherte Carrybit in Register
	rrf LOOP_2,F			;LOOP_2 wieder nach rechts rausschieben.
	decfsz LOOP_2,F 		;LOOP_2 um 1 verringern und prfen ob 0
	goto div32_loop         ;Counter ist noch nicht 0, zurck zur Schleife
	 
	rlf ZAHL_C0,F			;ZAEHLER_L		gesamter Zhler noch einmal nach links schieben, damit
	rlf ZAHL_C1,F			;ZAEHLER		das richtige Ergebnis im Zhler steht
	rlf ZAHL_C2,F			;ZAEHLER
	rlf ZAHL_C3,F			;ZAEHLER
	rlf ZAHL_C4,F			;ZAEHLER
	rlf ZAHL_C5,F			;ZAEHLER
	rlf ZAHL_C6,F			;ZAEHLER
	rlf ZAHL_C7,F			;ZAEHLER_H

	return
	
;*******************************************************************************************
; 32 bit Multiplikation: ZAHL_A x ZAHL_B = ZAHL_C
; Die Werte fr die Multiplikation mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) 
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
;*******************************************************************************************
mul32
	call clear_zahlc_p0		;Lschen des vorherigen Ergebnisses (ZAHL_C0 - ZAHL_C7)
	bsf ZAHL_C3,7			;um zu erkennen, wann 32 mal geschoben wurde
mul32_loop
	rrf ZAHL_A3,F			;verschiebe gesammtes Register Multiplikant1 um 1 nach rechts
	rrf ZAHL_A2,F			;
	rrf ZAHL_A1,F			;
	rrf ZAHL_A0,F			;wenn das LSB eine 1 ist wird das Carry gesetzt und es muss
	btfss STATUS,C			;eine Addition durchgefhrt werden ist das Carry = 0 entspricht
	goto mul32_rotate		;dies einer Multiplikation mit 0 und es wird nichts addiert

	movf ZAHL_B0,W			;Beginn der 32 bit Addition: Addiere Multiplikant2 _0 Register zu Ergebnis _4 Register
	addwf ZAHL_C4,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C5,F		;es ist ein bertrag vorhanden und das Ergebnis _5 Register wird um 1 erhht
	goto $+4
	incfsz ZAHL_C6,F		;wenn Ergebnis _5 Register berluft, erhhe Ergebnis _6 Register um 1
	goto $+2
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B1,W			;Addiere Multiplikant2 _1 Register zu Ergebnis _5 Register
	addwf ZAHL_C5,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C6,F		;es ist ein bertrag vorhanden und das Ergebnis _6 Register wird um 1 erhht
	goto $+2
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B2,W			;Addiere Multiplikant2 _2 Register zu Ergebnis _6 Register
	addwf ZAHL_C6,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B3,W			;Addiere Multiplikant2 _3 Register zu Ergebnis _7 Register
	addwf ZAHL_C7,F			;

mul32_rotate
	rrf ZAHL_C7,F			;Das 64 Bit Ergebnisregister wird um 1 nach rechts geschoben
	rrf ZAHL_C6,F
	rrf ZAHL_C5,F
	rrf ZAHL_C4,F
	rrf ZAHL_C3,F
	rrf ZAHL_C2,F
	rrf ZAHL_C1,F
	rrf ZAHL_C0,F
	btfss STATUS,C			;ist das Bit 7 von ZAHL_C3 im Carry angekommen wurde das Ergebnis
	goto mul32_loop			;insgesamt 32 mal rotiert und die Berechnung ist beendet
	return                  ;

;*******************************************************************************************
; 32 bit Addition: ZAHL_C + ZAHL_B = ZAHL_C
; Die Werte fr die Addition mssen in den Registern ZAHL_C0 (L) bis ZAHL_C3 (H)
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

add32
	movf ZAHL_B0,W			;Hole Low Byte Zahl2 ins W-Register
	addwf ZAHL_C0,F			;addiere Low Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C1,F		;es ist ein bertrag vorhanden und das L+1 Byte von Zahl2 wird um 1 erhht
	goto $+6				;kein bertrag beim L+1 Byte von Zahl2, 
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto $+4				;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B1,W			;Hole L+1 Byte Zahl2 ins W-Register
	addwf ZAHL_C1,F			;addiere L+1 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto $+4				;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B2,W			;Hole L+2 Byte Zahl2 ins W-Register
	addwf ZAHL_C2,F			;addiere L+2 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B3,W			;Hole High Byte Zahl2 ins W-Register
	addwf ZAHL_C3,F			;addiere High Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht
	return

;*******************************************************************************************
; 32 bit Subtraktion: ZAHL_C - ZAHL_B = ZAHL_C
; Die Werte fr die Subtraktion mssen in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
; und ZAHL_C0 (L) bis ZAHL_C4 (H) stehen, wobei ZAHL_B von ZAHL_C subtrahiert wird 
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

sub32
	movfw ZAHL_B0			;Hole 0/7-Byte von B nach W
	subwf ZAHL_C0,F			;Subtrahiere B von C, 0/7-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 8/15-Byte
	subwf ZAHL_C1,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_2
	movfw ZAHL_B1			;Hole 8/15-Byte von B nach W
	subwf ZAHL_C1,F			;Subtrahiere B von C, 8/15-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_3
	movfw ZAHL_B2			;Hole 16/23-Byte von B nach W
	subwf ZAHL_C2,F			;Subtrahiere B von C, 16/23-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_4
	movfw ZAHL_B3			;Hole 24/31-Byte von B nach W
	subwf ZAHL_C3,F			;Subtrahiere B von C, 24/31-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_end			;Ergebnis ist positiv (Carry Bit = H) weiter zum Ende des Unterprogramms
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_end
	return


runden_div32					;Unterprogramm zum ganzzahligen Runden des Ergebnisses der Division aus div32
	bcf STATUS,C				;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf ZAHL_B0					;ZAHL_B 1x nach links schieben, d.h. mit 2 malnehmen 
	rlf ZAHL_B1
	rlf ZAHL_B2
	rlf ZAHL_B3
	btfsc STATUS,C
	goto runden_div32_inc	

	movfw ZAHL_A3				;wenn Rest >= ZAHL_A/2 ist, d.h. >=0,5 ist, erhhe ZAHL_C um 1
	subwf ZAHL_B3,w				;subtrahiere ZAHL_A/2 von Rest
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Ergebnis negativ (STATUS,C=0), Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0  (STATUS,Z=1)
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A2				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A2 ins W-Reg.
	subwf ZAHL_B2,w				;subtrahiere W-Reg. von ZAHL_B2
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A1				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A1 ins W-Reg.
	subwf ZAHL_B1,w				;subtrahiere W-Reg. von ZAHL_B1
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A0				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A0 ins W-Reg.
	subwf ZAHL_B0,w				;subtrahiere W-Reg. von ZAHL_B0
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Rest (ZAHL_B) < ZAHL_A/2, Rcksprung aus dem Unterprogramm
runden_div32_inc	
	incfsz ZAHL_C0				;Rest (ZAHL_B) >= ZAHL_A/2, erhhe ZAHL_C0 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C1				;berlauf vorhanden,  erhhe ZAHL_C1 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C2				;berlauf vorhanden,  erhhe ZAHL_C2 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incf ZAHL_C3				;berlauf vorhanden,  erhhe ZAHL_C3 um 1
	return						;Rcksprung aus dem Unterprogramm


convert_hex_dig				;Unterprogramm zum Zerlegen einer 24-Bit Zahl in die einzelnen Digits (max.999.999)zur Anzeige auf dem Display
	;clrf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7

							;von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	clrf ZAHL_B0			;Lade ZAHL_B mit 1.000.000.000 (0x3B9ACA00)
	movlw 0xCA				;
	movwf ZAHL_B1
	movlw 0x9A				;
	movwf ZAHL_B2
	movlw 0x3B				;
	movwf ZAHL_B3
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-3				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	call add32				;   und letzte Subtraktion rckgngig machen

							;von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	clrf ZAHL_B0			;Lade ZAHL_B mit 100.000.000 (0x05F5E100)
	movlw 0xE1				;
	movwf ZAHL_B1
	movlw 0xF5				;
	movwf ZAHL_B2
	movlw 0x05				;
	movwf ZAHL_B3
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-3				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_7				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x80				;Lade ZAHL_B mit 10.000.000 (0x00989680)
	movwf ZAHL_B0
	movlw 0x96				;
	movwf ZAHL_B1
	movlw 0x98				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_7				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_7				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_6				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x40				;Lade ZAHL_B mit 1.000.000 (0x000F4240)
	movwf ZAHL_B0
	movlw 0x42				;
	movwf ZAHL_B1
	movlw 0x0F				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_6				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_6				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_5				;Lsche Register, von der 24-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	incf DISP_5				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_5				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_4				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	incf DISP_4				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C2 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_4				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_3				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	incf DISP_3				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_3				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_2				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	incf DISP_2				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_2				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf DISP_1				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	incf DISP_1				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf DISP_1				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	movfw ZAHL_C0			;
	movwf DISP_0
	return

convert_dig_hex				;Unterprogramm zum Umwandeln der einzelnen Digits (max.999.999) in eine 24-Bit Zahl
	call clear_zahlc_p0		;Lsche Additionsregister ZAHL_C
	movfw DISP_7			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex6		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x80				;Lade ZAHL_B mit 10.000.000 (0x00989680)
	movwf ZAHL_B0
	movlw 0x96				;
	movwf ZAHL_B1
	movlw 0x98				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_7			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex6
	movfw DISP_6			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex5		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x40				;Lade ZAHL_B mit 1.000.000 (0x000F4240)
	movwf ZAHL_B0
	movlw 0x42				;
	movwf ZAHL_B1
	movlw 0x0F				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_6			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex5
	movfw DISP_5			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex4		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_5			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex4
	movfw DISP_4			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex3		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_4			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex3
	movfw DISP_3			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex2		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_3			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex2
	movfw DISP_2			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex1		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_2			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex1
	movfw DISP_1			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex0		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz DISP_1			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex0
	call clear_zahlb_p0		;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	movfw DISP_0			;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	return					;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movwf ZAHL_B0			;Speichere Digit im Additionsregister
	call add32				;Addiere ZAHL_B zu ZAHL_C
	return

clear_zahla_p0			;Unterprogramm zum Lschen der ZAHL_A Register, Page 0
	clrf ZAHL_A0		;
	clrf ZAHL_A1		;
	clrf ZAHL_A2		;
	clrf ZAHL_A3		;
	return

clear_zahlb_p0			;Unterprogramm zum Lschen der ZAHL_B Register, Page 0
	clrf ZAHL_B0		;
	clrf ZAHL_B1		;
	clrf ZAHL_B2		;
	clrf ZAHL_B3		;
	return

clear_zahlc_p0			;Unterprogramm zum Lschen der ZAHL_C Register, Page 0
	clrf ZAHL_C0		;
	clrf ZAHL_C1		;
	clrf ZAHL_C2		;
	clrf ZAHL_C3		;
	clrf ZAHL_C4		;
	clrf ZAHL_C5		;
	clrf ZAHL_C6		;
	clrf ZAHL_C7		;
	return


	ORG 0x7F6

menue_tab
	addwf PCL,f
	nop
	goto menu_zalt_set		; Aufruf des Programmteils Zhlerstnde Vortage/Vormonat
	goto menu_zp_set		; Aufruf des Programmteils Einstellung Zhler Produktion
	goto menu_datuz_set		; Aufruf des Programmteils Einstellung Datum/Uhrzeit
	goto menu_testrx_set	; Aufruf des Programmteils menu_testrx_set
	goto menu_regpara_set	; Aufruf des Programmteils Einstellung Reglerparameter
	goto menu_testpwm_set	; Aufruf des Programmteils menu_testpwm_set
	goto menu_p_kenn_set	; Aufruf des Programmteils menu_p_kenn_set	
	goto menu_funct_rb2_set	; Menpunkt 8: Funktion des Ausgangs RB2 (Pin8): Ausgang On/Off oder Ausgang Tx Pb/e

;*******************************************************************************************
;	PAGE 1, 	bsf PCLATH,3				;Select Page 1
;				bcf PCLATH,4
;*******************************************************************************************
	ORG 0x800					;PAGE 1

disp_menue_1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse	
	movfw MENU_COUNT		;Schreiben der Menpunkt Nr.
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen

	movlw 0x08				;Programmzhler fuer Menpunkt tabelle im Bereich 0x0800..0x08FF 
	movwf PCLATH
	movfw MENU_COUNT
	addwf PCL,f
	nop
	goto dispm1_zalt		;Menpunkt 1
	goto dispm1_zp			;Menpunkt 2
	goto dispm1_datuz		;Menpunkt 3
	goto dispm1_testrx		;Menpunkt 4
	goto dispm1_regpara		;Menpunkt 5
	goto dispm1_testpwm		;Menpunkt 6
	goto dispm1_p_kenn		;Menpunkt 7
	goto dispm1_f_rb2		;Menpunkt 8


dispm1_zalt					;Menpunkt 1
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'l'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'd'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end	

dispm1_zp					;Menpunkt 2
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'l'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'd'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_datuz				;Menpunkt 3
	movlw A'D'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'/'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'U'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_testrx				;Menpunkt 4
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'R'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'x'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_regpara				;Menpunkt 5, 
	movlw A'R'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'g'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'l'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_testpwm				;Menpunkt 6
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'W'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'M'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_p_kenn				;Menpunkt 7
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'K'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'g'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto dispm1_end

dispm1_f_rb2
	movlw A'F'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'k'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'8'				;ASCII:
	call disp_daten			;bertrage das Zeichen

dispm1_end
	call wait_250_p1		;Wartezeit 250ms Page 1
	bcf MENU_STAT,7			;lsche das Status Bit
	return


disp_start
	movlw 0x82				;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'H'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'M'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'6'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'9'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'1'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return


disp_main
	bcf STAT_TIMER,2		;lsche Statusbit fr 1 Sekunde
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse

	movlw 0x03				;Wenn der Displayzhler =3, weiter bei Anzeige Datum/Uhr
	subwf DISP_NR,w
	btfsc STATUS,Z
	goto disp_dat_uhr

	movlw 0x02				;Wenn der Displayzhler =2, weiter bei Anzeige P bezug / P einspeisung und Regler
	subwf DISP_NR,w
	btfsc STATUS,Z
	goto disp_regler

	movlw 0x01				;Wenn der Displayzhler =1, weiter bei Anzeige Produktion: Zhler und Leistung
	subwf DISP_NR,w
	btfsc STATUS,Z
	goto disp_prod

	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'b'				;ASCII: b (Bezug)
	call disp_daten			;bertrage das Zeichen
	movfw ZAEHL_BEZUG_0
	movwf ZAHL_C0
	movfw ZAEHL_BEZUG_1
	movwf ZAHL_C1
	movfw ZAEHL_BEZUG_2
	movwf ZAHL_C2
	movfw ZAEHL_BEZUG_3
	movwf ZAHL_C3
	call disp_zdat
	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII: b (Bezug)
	call disp_daten			;bertrage das Zeichen
	movfw ZAEHL_EINSP_0
	movwf ZAHL_C0
	movfw ZAEHL_EINSP_1
	movwf ZAHL_C1
	movfw ZAEHL_EINSP_2
	movwf ZAHL_C2
	movfw ZAEHL_EINSP_3
	movwf ZAHL_C3
	call disp_zdat
	return

disp_prod
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII: b (Bezug)
	call disp_daten			;bertrage das Zeichen
	movfw ZAEHL_PROD_0
	movwf ZAHL_C0
	movfw ZAEHL_PROD_1
	movwf ZAHL_C1
	movfw ZAEHL_PROD_2
	movwf ZAHL_C2
	movfw ZAEHL_PROD_3
	movwf ZAHL_C3
	call disp_zdat
	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII: p (Produktion)
	call disp_daten			;bertrage das Zeichen
	movfw P_PROD_L
	movwf ZAHL_C0
	movfw P_PROD_H
	movwf ZAHL_C1
	clrf ZAHL_C2
	call disp_pdat
	return

disp_regler
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII: e (Einspeisung)
	btfss P_INOUT_H,7
	movlw A'b'				;ASCII: b (Bezug)
	call disp_daten			;bertrage das Zeichen
	movfw P_INOUT_L
	movwf ZAHL_C0
	movfw P_INOUT_M
	movwf ZAHL_C1
	movfw P_INOUT_H
	movwf ZAHL_C2
	bcf ZAHL_C2,7
	call disp_pdat

	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'16'				;hole Wert 16 nach W
	btfss STAT_PWM,0		;prfen, ob Bit fr "Regler Aus" gesetzt ist
	goto $+3				;
	call disp_blank_x		;Bit fr "Regler Aus" ist gesetzt, lsche 2te Zeile (16x blank)
	return					;Rcksprung

	movlw A'R'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movfw REGOUT_DISP
	call convdisp_hex_3dig	;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A'%'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	btfsc STAT_PWM,1
	goto disp_fb
	btfsc STAT_PWM,2
	goto disp_fk
	movlw d'20'
	subwf T_H,w
	btfsc STATUS,C
	goto t_positiv			;positiv
	movlw A'-'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw d'20'
	movwf DISP_0
	movfw T_H
	subwf DISP_0,f
	goto t_disp
t_positiv
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw d'20'
	subwf T_H,w
	movwf DISP_0
t_disp
	clrf DISP_1
	bcf STATUS,C
	rrf DISP_0
	rrf DISP_1	
	movfw DISP_0
	call convdisp_hex_3dig	;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A','				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'5'				;ASCII:
	btfss DISP_1,7
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return
disp_fb
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'F'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'b'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	return
disp_fk
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'F'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'k'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	return


disp_dat_uhr
	movlw d'3'
	call disp_blank_x		;Unterprogramm zum Schreiben von x-mal "blank", x ist der Wert in W
	movfw TAG				;
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw MONAT				;
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'2'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw JAHR				;
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw d'3'
	call disp_blank_x		;Unterprogramm zum Schreiben von x-mal "blank", x ist der Wert in W

	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'4'
	call disp_blank_x		;Unterprogramm zum Schreiben von x-mal "blank", x ist der Wert in W
	movfw STUNDE				;
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw MINUTE			;
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw SEKUNDE			;
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw d'4'
	call disp_blank_x		;Unterprogramm zum Schreiben von x-mal "blank", x ist der Wert in W
	return

disp_zdat
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	bcf PCLATH,3			;Select Page 0
	call convert_hex_dig	;Unterprogramm zum Zerlegen einer 24-Bit Zahl in die einzelnen Digits (max.999.999)zur Anzeige auf dem Display
	bsf PCLATH,3			;Select Page 1	
disp_zaehler
	movfw DISP_7			;kWh 100.000er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_6			;kWh 10.000er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_5			;kWh 1.000er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_4			;kWh 100er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_3			;kWh 10er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_2			;kWh 1er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A','				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw DISP_1			;kWh 0,1er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_0			;kWh 0,01er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'k'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'W'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return

disp_pdat
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	clrf ZAHL_C3
	bcf PCLATH,3			;Select Page 0
	call convert_hex_dig	;Unterprogramm zum Zerlegen einer 24-Bit Zahl in die einzelnen Digits (max.999.999)zur Anzeige auf dem Display
	bsf PCLATH,3			;Select Page 1	
	movfw DISP_5			;Leistung 100.000er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_4			;Leistung 10.000er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_3			;Leistung 1.000er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_2			;Leistung 100er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_1			;Leistung 10er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw DISP_0			;Leistung 1er, untere 4Bit
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'W'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw d'5'
	call disp_blank_x		;Unterprogramm zum Schreiben von x-mal "blank", x ist der Wert in W
	return


disp_set_zalt				;Menpunkt 1: Anzeige des Tages -X oder Vormonat
	call clear_display		;Display lschen
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'14'
	subwf TEMP_11,w
	btfsc STATUS,Z
	goto disp_set_vm
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'g'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'-'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_11
	addlw 0x01
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	return
disp_set_vm
	movlw A'V'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return

disp_set_zalt1				;Anzeige Zhler Bezug/Einspeisung oder Produktion
	call clear_display		;Display lschen
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	btfsc TEMP_12,0			;prfen ob 0 oder 1
	goto disp_set_zprod		;1, weiter bei Anzeige Zhler Produktion
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'b'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_11
	call readEE
	movwf ZAHL_C3
	movfw TEMP_11
	addlw 0x01
	call readEE
	movwf ZAHL_C2
	movfw TEMP_11
	addlw 0x02
	call readEE
	movwf ZAHL_C1
	movfw TEMP_11
	addlw 0x03
	call readEE
	movwf ZAHL_C0
	call disp_zdat			;Unterprogramm zur Anzeige der 8 Digits fr Zhlerstnde

	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_11
	addlw 0x04
	call readEE
	movwf ZAHL_C3
	movfw TEMP_11
	addlw 0x05
	call readEE
	movwf ZAHL_C2
	movfw TEMP_11
	addlw 0x06
	call readEE
	movwf ZAHL_C1
	movfw TEMP_11
	addlw 0x07
	call readEE
	movwf ZAHL_C0	
	call disp_zdat			;Unterprogramm zur Anzeige der 8 Digits fr Zhlerstnde
	return

disp_set_zprod				;Anzeige Zhler Produktion
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_11
	addlw 0x08
	call readEE
	movwf ZAHL_C3
	movfw TEMP_11
	addlw 0x09
	call readEE
	movwf ZAHL_C2
	movfw TEMP_11
	addlw 0x0A
	call readEE
	movwf ZAHL_C1
	movfw TEMP_11
	addlw 0x0B
	call readEE
	movwf ZAHL_C0	
	call disp_zdat			;Unterprogramm zur Anzeige der 8 Digits fr Zhlerstnde
	return


disp_set_zp
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'Z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_zaehler
	movlw 0xC3				;Zeichen 4 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_11			;Imp/kWh (x1000) 10000er + 1000er
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'I'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'/'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'k'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'W'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	return

disp_set_datuz
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movfw ZAHL_B0			;untere 4 bit, Tag 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_B1			;untere 4 bit, Tag 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_B2			;untere 4 bit, Monat 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_B3			;untere 4 bit, Monat 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'2'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_C0			;untere 4 bit, Jahr 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_C1			;untere 4 bit, Jahr 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display

	movlw 0xC0				;Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'h'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'r'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw d'3'
	call disp_blank_x		;Unterprogramm zum Schreiben von x-mal "blank", x ist der Wert in W
	movfw ZAHL_C2			;untere 4 bit, Stunde 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_C3			;untere 4 bit, Stunde 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_C4			;untere 4 bit, Minute 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_C5			;untere 4 bit, Minute 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'0'				;ASCII: Sekunde 10er
	call disp_daten			;bertrage das Zeichen
	movlw A'0'				;ASCII: Sekunde 1er
	call disp_daten			;bertrage das Zeichen
	return

disp_set_testrx
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'B'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'y'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	btfsc MENU_STAT,1		;prfen ob bit fr bytes 160-319 nicht gesetzt ist
	goto disp_set_testrx1	;bit ist gesetzt, weiter bei Anzeige bytes 160-319
	movlw A'0'
	call disp_daten			;bertrage das Zeichen
	movlw A'0'
	call disp_daten			;bertrage das Zeichen
	movlw A'0'
	call disp_daten			;bertrage das Zeichen
	movlw A'-'
	call disp_daten			;bertrage das Zeichen
	movlw A'2'
	call disp_daten			;bertrage das Zeichen
	movlw A'3'
	call disp_daten			;bertrage das Zeichen
	movlw A'9'
	call disp_daten			;bertrage das Zeichen
	goto disp_set_testrx_end
disp_set_testrx1			;Anzeige bytes 160-319
	movlw A'2'
	call disp_daten			;bertrage das Zeichen
	movlw A'4'
	call disp_daten			;bertrage das Zeichen
	movlw A'0'
	call disp_daten			;bertrage das Zeichen
	movlw A'-'
	call disp_daten			;bertrage das Zeichen
	movlw A'4'
	call disp_daten			;bertrage das Zeichen
	movlw A'7'
	call disp_daten			;bertrage das Zeichen
	movlw A'9'
	call disp_daten			;bertrage das Zeichen
disp_set_testrx_end
	bcf MENU_STAT,0
	call wait_250_p1		;Wartezeit 250ms Page 1
	return

disp_ausw_testrx
	call clear_display		;Unterprogramm zum Lschen des Display
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movfw TEMP_11			;
	btfsc MENU_STAT,1		;0 = Daten 0-14, 1 = Daten 15-29,
	addlw d'15'
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	call wait_250_p1		;Wartezeit 250ms Page 1, Wartezeit von 4x250ms = 1s
	call wait_250_p1		;Wartezeit 250ms Page 1
	call wait_250_p1		;Wartezeit 250ms Page 1
	call wait_250_p1		;Wartezeit 250ms Page 1
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse

	movfw TEMP_11			;kopiere den Blockwert (0-14) in zustzliches Temp Register
	movwf TEMP_12
	movlw d'5'				;prfen, ob erste 5 Blcke (Bank1, Adresse 0x0A0..0x0EF) gewhlt sind
	subwf TEMP_11,w
	btfsc STATUS,C
	goto $+5				;Ergebnis >=5, weiter bei nchster Prfung
	movlw 0xA0				;Ergebnis <5, d.h. untere 5 Blcke, lade Adresse 0xA0 in FSR 0 Register
	movwf FSR0
	clrf FSR0H				;lsche IRP Bit fr Bank 0/1
	goto multi_x16			;weiter bei Multiplikation mit 16

	movlw d'10'				;prfen, ob 2te 5 Blcke (Bank 2, Adresse 0x020..0x06F) gewhlt sind
	subwf TEMP_11,w
	btfsc STATUS,C
	goto $+7				;Ergebnis >=10, d.h. 3te 5 Blcke
	movlw 0x20				;Ergebnis <10, d.h. 2te 5 Blcke, lade Adresse 0x20 in FSR 0 Register
	movwf FSR0
	bsf FSR0H,0				;setze IRP Bit fr Bank 2/3
	movlw d'5'				;subtrahiere 5 vom Blockwert
	subwf TEMP_12
	goto multi_x16			;weiter bei Multiplikation mit 16

	movlw 0xA0				;Ergebnis >=10, d.h. 3te 5 Blcke, lade Adresse 0xA0 in FSR 0 Register
	movwf FSR0	
	bsf FSR0H,0				;setze IRP Bit fr Bank 2/3
	movlw d'10'				;subtrahiere 10 vom Blockwert
	subwf TEMP_12

multi_x16
	swapf TEMP_12			;vertausche obere und untere 4 Bit (Multiplikation mit 16)
	movfw TEMP_12			;den mit 16 multiplizierten Wert zum FSR 0 Register addieren
	addwf FSR0

	movlw d'8'				;in der ersten Zeile sollen 8 Byte im Hex Format angezeigt werden
	movwf TEMP_12
	moviw FSR0++			;hole den Wert nach W und erhhe FSR0 um 1
	call disp_hex			;Unterprogrammm zur Anzeige eines 8 bit Zeichens im hex Format
	decfsz TEMP_12			;reduziere Byte Zhler um 1 und prfe ob 0
	goto $-3				;Zhler noch nicht 0, Anzeige nchstes Byte

	movlw 0xC0				;8 Byte in Zeile 1 bertragen, Zeichen 1 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse	
	movlw d'8'				;in der zweiten Zeile sollen 8 Byte im Hex Format angezeigt werden
	movwf TEMP_12
	moviw FSR0++			;hole den Wert nach W und erhhe FSR0 um 1
	call disp_hex			;Unterprogrammm zur Anzeige eines 8 bit Zeichens im hex Format
	decfsz TEMP_12			;reduziere Byte Zhler um 1 und prfe ob 0
	goto $-3				;Zhler noch nicht 0, Anzeige nchstes Byte
	bcf MENU_STAT,0
	return

disp_set_regpara			;Zeile 1: Kp:0,XX_Tn:XXs__
	bcf MENU_STAT,3
	movlw d'10'
	subwf M_HOR_CNT,w
	btfsc STATUS,C
	bsf MENU_STAT,3
	btfsc MENU_STAT,3
	goto disp_regpara_1

	movlw 0x80				;Zeichen 1 in der 1. Zeile:  Ein_Pl:XXXXW__
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	btfsc MENU_STAT,2
	goto $+8
	movlw A'E'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto $+7
	movlw A'A'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'l'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_A0			;untere 4 bit, Tn, 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_A1			;untere 4 bit, Tn, 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_A2			;untere 4 bit, Tn, 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_A3			;untere 4 bit, Tn, 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A'W'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
disp_regpara_1				;Zeile 2 bzw. Zeile 1: 
	movlw 0xC0				;Zeichen 1 in der 2. Zeile : Kp:0,XX_Tn:XXs
	btfsc MENU_STAT,3		;prfen ob Bit fr 1.Zeile nicht gesetzt ist
	movlw 0x80				;Zeichen 1 in der 1. Zeile : Kp:0,XX_Tn:XXs
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'K'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A','				;ASCII:
	call disp_daten			;bertrage das Zeichen	
	movfw ZAHL_B0			;untere 4 bit, Kp, 0,1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_B1			;untere 4 bit, Kp, 0,01er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_B2			;untere 4 bit, Tn, 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_B3			;untere 4 bit, Tn, 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	btfss MENU_STAT,3		;prfen ob Bit fr 1.Zeile nicht gesetzt ist
	return

	movlw 0xC0				;Zeichen 1 in der 2. Zeile: Tm:XXC_ta:XXmin
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'T'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_C0			;untere 4 bit, tmax, 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_C1			;untere 4 bit, tmax, 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw 0xDF				;ASCII:  (Grad)
	call disp_daten			;bertrage das Zeichen
	movlw A'C'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_C2			;untere 4 bit, tmax, 10er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movfw ZAHL_C3			;untere 4 bit, tmax, 1er
	call disp_zahl			;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return


disp_set_testpwm			;Menpunkt 6
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'W'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'M'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'u'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A't'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movfw REGOUT_DISP
	call convdisp_hex_2dig	;Unterprogramm zum Zerlegen einer 2-stelligen Zahl in einzene Digits + Anzeige auf dem Display
	movlw A'%'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	bcf MENU_STAT,0
	call wait_250_p1		;Wartezeit 250ms Page 1
	return

disp_set_p_kenn				;Menpunkt 7
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'-'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'K'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'z'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	movfw ZAHL_C0			;obere 4Bit Byte 1
	call disp_hex2			;Umwandeln von Dezimal in hex plus Anzeige auf dem Display
	movfw ZAHL_C1			;untere 4Bit Byte 1
	call disp_hex2			;Umwandeln von Dezimal in hex plus Anzeige auf dem Display
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_C2			;obere 4Bit Byte 2
	call disp_hex2			;Umwandeln von Dezimal in hex plus Anzeige auf dem Display
	movfw ZAHL_C3			;untere 4Bit Byte 2
	call disp_hex2			;Umwandeln von Dezimal in hex plus Anzeige auf dem Display
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw ZAHL_C4			;obere 4Bit Byte 3
	call disp_hex2			;Umwandeln von Dezimal in hex plus Anzeige auf dem Display
	movfw ZAHL_C5			;untere 4Bit Byte 3
	call disp_hex2			;Umwandeln von Dezimal in hex plus Anzeige auf dem Display
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return

disp_set_frb2
	movlw 0x80				;Zeichen 1 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'i'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'8'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	btfss MENU_STAT,3
	goto $+0x0A
	movlw A'P'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'b'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'/'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	goto $+0x08
	movlw A'0'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'/'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'1'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	call disp_blank			;Unterprogramm zum Schreiben von "blank"
	bcf MENU_STAT,0
	call wait_250_p1		;Wartezeit 250ms Page 1
	return

disp_blank					;Unterprogramm zum Schreiben von "blank"
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	return

disp_blank_x				;Schreiben von "blank" x mal entsprechend dem wert in W
	movwf LOOP_1			;kopiere W nach Loop_1
	movlw 0x20				;Zeichen fr "blank"
	call disp_daten			;bertrage das Zeichen
	decfsz LOOP_1			;reduziere Zhler um 1 und prfe ob null
	goto $-3				;Zhler noch nicht Null, zurck zur Schleife
	return					;Zhler = Null, fertig


;*******************************************************************************************
;	Unterprogramme Initialisierung/Ansteuerung des Display
;*******************************************************************************************



init_display					;Unterprogramm zur Initialisierung des Displays
	call wait_250_p1			;Wartezeit 250ms Page 1
	movlw 0x38					;Befehl: Function Set
	call disp_befehl	;
	movlw 0x39					;Befehl: Function Set, (muss zweimal bertragen werden)
	call disp_befehl	;
	movlw 0x14					;Befehl: Bias Set
	call disp_befehl	;
	movlw 0x78					;Befehl: Contrast Set
	call disp_befehl			;
	movlw 0x52					;Befehl: Power/Icon/Contrast Contr.
	call disp_befehl			;	
	movlw 0x69					;Befehl: Follower Control
	call disp_befehl			;
	movlw 0x0C					;Befehl: Display On/Off Control (00001100)
	call disp_befehl			;
	call clear_display			;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x06					;Befehl: Entry Mode Set (00000110)
	call disp_befehl			;
	return

clear_display					;Unterprogramm zum lschen des Displays					
	movlw 0x01					;Befehl: Clear Display
	call disp_befehl	;
	movlw d'7'					;geforderte Wartezeit des Displays von 7ms
	call wait 		
	return

cursor_off
	movlw 0x0C					;Befehl: Display On/Off Control, Cursor OFF (00001100)
	goto $+2
cursor_on
	movlw 0x0E					;Befehl: Display On/Off Control, Cursor ON (00001110)
	call disp_befehl			;
	movlw d'20'					;geforderte Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

disp_befehl
	movwf SHIFT_REG				;Register mit den Daten, die ausgegeben werden sollen
	bcf DISP_EN					;Enable auf low setzen
	bcf DISP_RS					;RS auf Low setzen, da es sich um ein Kommando handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_daten
	movwf SHIFT_REG				;Das W-Register wird in das Reg. SHIFT_REG kopiert
	bcf DISP_EN					;Enable auf low setzen
	bsf DISP_RS					;RS auf High setzen, da es sich um ein Zeichen handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_routine
	bcf SI_CLK					;Takt auf Low setzen, damit die Daten bei der steigenden Flanke bernommen werden knnen
	movlw d'8'					;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT				;Zhler fr die bertragenen bits
shiftDaten
	rlf SHIFT_REG,F				;Das Zeichen wird nach links verschoben. Das Carryflag wird an den Ausgang weitergegeben
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bsf SI_DAT					;Carryflag = 1 -> Pin SI_DAT auf Highpegel setzen
	btfss STATUS,C				;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf SI_DAT					;Carryflag = 0 -> Pin SI_DAT auf Lowpegel setzen
	nop							;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	nop
	bsf SI_CLK					;mit dieser steigenden Flanke werden die Daten bernommen
	nop							;kurze Wartezeit
	nop
	bcf SI_CLK					;negative Flanke des Taktes
	decfsz SHIFT_CNT,F			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto shiftDaten				;nchstes bit bertragen
	bsf DISP_EN					;Enable auf high setzen, mit der positiven Flanke werden Daten bertragen 
	bcf DISP_RS					;bertragung beendet Pin RS auf Low zurcksetzen
	bcf SI_DAT					;Pin SI_DAT auf Low zurcksetzen
	movlw d'30'					;geforderte Wartezeit (min. 26,3 us)
	movwf SHIFT_CNT
	decfsz SHIFT_CNT
	goto $-1
	return						;Rcksprung aus dem Unterprogramm


;*******************************************************************************************
;	Weitere Unterprogramme zur Anzeige auf dem Display
;*******************************************************************************************

disp_zahl						;Unterprogramm zur Anzeige einer Zahl (0...9 + 0x30) auf dem Display
	addlw 0x30					;addiere obere 4Bit
	call disp_daten				;bertrage das Zeichen
	return

convdisp_hex_2dig				;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

convdisp_hex_3dig				;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


convert_dig					;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf TEMP_15			;Temp Register fr Zwischenspeicherung des zu zerlegenden Wertes
	clrf TEMP_13			;lsche Temp Register
	clrf TEMP_14			;lsche Temp Register
dig_100er
	movlw d'100'			;schreibe Wert 100 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 100 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_10er			;Ergebnis negativ, weiter bei 10er
	incf TEMP_13			;Ergebnis noch positiv, Erhhe 100er um 1 
	movlw d'100'			; und reduziere den zu zerlegenden Werte um 100 im F-Register
	subwf TEMP_15,f			;
	goto dig_100er			;Zurck zur Schleife
dig_10er
	movlw d'10'				;schreibe Wert 10 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 10 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_1er			;Ergebnis negativ, weiter bei 1er
	incf TEMP_14			;Ergebnis noch positiv, Erhhe 10er um 1 
	movlw d'10'				; und reduziere den zu zerlegenden Werte um 10 im F-Register
	subwf TEMP_15,f			;
	goto dig_10er			;Zurck zur Schleife
dig_1er
	movfw TEMP_15
	return


disp_hex					;Unterprogrammm zur Anzeige eines 8 bit Zeichens im hex Format
	call trennen_hex		;auftrennen einer 8 bit Zahl in obere (Temp 14) und unter 4Bit (Temp 13)
	movfw TEMP_14			;obere 4 Bit
	call disp_hex2
	movfw TEMP_13			;untere 4 Bit
	call disp_hex2	
	return

disp_hex2
	movwf TEMP_15
	movlw 0x0A				;prfen, ob untere 4 bit <10
	subwf TEMP_15,w
	movfw TEMP_15
	btfss STATUS,C
	addlw 0x30				;untere 4 bit <10 (0x00-0x09), addiere 0x30
	btfsc STATUS,C	
	addlw 0x37				;untere 4 bit >=10 (0x0A-0x0F), addiere 0x37			
	call disp_daten			;bertrage das Zeichen 
	return

trennen_hex					;auftrennen einer 8 bit Zahl in obere (Temp 14) und unter 4Bit (Temp 13)
	movwf TEMP_13			;kopiere W in Temp registter
	movwf TEMP_14
	swapf TEMP_14			;vertausche obere und untere 4 Bit
	bcf TEMP_13,7			;lsche obere 4 Bit
	bcf TEMP_13,6
	bcf TEMP_13,5
	bcf TEMP_13,4
	bcf TEMP_14,7			;lsche obere 4 Bit
	bcf TEMP_14,6
	bcf TEMP_14,5
	bcf TEMP_14,4
	return	

clear_zahlc_p1			;Unterprogramm zum Lschen der ZAHL_C Register
	clrf ZAHL_C0		;
	clrf ZAHL_C1		;
	clrf ZAHL_C2		;
	clrf ZAHL_C3		;
	clrf ZAHL_C4		;
	clrf ZAHL_C5		;
	clrf ZAHL_C6		;
	clrf ZAHL_C7		;
	return

wait_250_p1					;Unterprogramm fr Wartezeit 250ms Page1
	movlw d'250'
wait						;Unterprogramm fr Wartezeit Xms Page1
	movwf LOOP_1
loop_250
	movlw d'250'			;loop_10 mu 4 Takten entsprechen fr 1ms: 4MHz /4/250/4Takte
	movwf LOOP_2
loop_10
	nop
	decfsz LOOP_2
	goto loop_10
	decfsz LOOP_1
	goto loop_250
	return

lade_zaehl_dat
	movlw 0x02		;Startbytes: 77 07 01 00, setze FSR 1H auf 2
	movwf FSR1H
	movlw 0x20		;setze FSR1 auf Adresse 0xA0
	movwf FSR1
	movlw 0x77		;hole Wert 0x77 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x07		;hole Wert 0x07 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x01		;hole Wert 0x01 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x00		;hole Wert 0x00 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1

	movlw 0x01		;Zhler Bezug: 01 08 00 FF, hole Wert 0x01 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x08		;hole Wert 0x08 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x00		;hole Wert 0x00 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0xFF		;hole Wert 0xFF nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1

	movlw 0x02		;Zhler Lieferung: 02 08 00 FF, hole Wert 0x02 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x08		;hole Wert 0x08 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x00		;hole Wert 0x00 nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0xFF		;hole Wert 0xFF nach W
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1

	movlw 0x09		;Leistung: 10 07 00 FF,  EEPROM Adresse: hole Wert 0x09 nach W
	call readEE		;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x0A		;EEPROM Adresse: hole Wert 0x0A nach W
	call readEE		;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0x0B		;EEPROM Adresse: hole Wert 0x0B nach W
	call readEE		;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwi FSR1++	;schiebe Wert nach INDF1 und erhhe FSR1 um 1
	movlw 0xFF		;hole Wert 0xFF nach W
	movwf INDF1		;schiebe Wert nach INDF1

	movlw 0x20
	movwf FSR1
	return

inc_datum
	incf TAG				;erhhe Tag
	movlw 0x02				;prfen, ob Monat = Februar
	subwf MONAT,w
	btfss STATUS,Z
	goto inc_datum1			;Monat nicht Februar
	btfsc JAHR,0			;Monat ist Februar, prfen, ob Schaltjahr => Bit 0+1 = 0
	goto inc_datum1			;Bit0 = 1 ,d.h. kein Schaltjahr
	btfsc JAHR,1			;
	goto inc_datum1			;Bit1 = 1 ,d.h. kein Schaltjahr
	movlw 0x1D				;Jahr ist Schaltjahr, hole Wert 29 nach W
	goto inc_datum2
inc_datum1
	movlw 0x0E				;vorbereiten fuer Monatstage-Tabelle im Bereich 0x0E00..0x0EFF
	movwf PCLATH			;
	movfw MONAT
	addlw 0xFF
	call monatstage_tab		;Datentabelle fr die Anzahl Tage des Monats
inc_datum2
	addlw 0x01
	subwf TAG,w
	btfss STATUS,C
	return
	movlw 0x01
	movwf TAG
	incf MONAT

	movlw 0x0D				;hole Wert 13 nach W
	subwf MONAT,w
	btfss STATUS,C
	return
	movlw 0x01
	movwf MONAT
	incf JAHR
	return

;*******************************************************************************************
;	Weitere Unterprogramme zur Mensteuerung
;*******************************************************************************************

convert_datuz
	movfw TAG
	call convert_dig
	movwf ZAHL_B1
	movfw TEMP_14
	movwf ZAHL_B0
	movfw MONAT
	call convert_dig
	movwf ZAHL_B3
	movfw TEMP_14
	movwf ZAHL_B2
	movfw JAHR
	call convert_dig
	movwf ZAHL_C1
	movfw TEMP_14
	movwf ZAHL_C0
	movfw STUNDE
	call convert_dig
	movwf ZAHL_C3
	movfw TEMP_14
	movwf ZAHL_C2
	movfw MINUTE
	call convert_dig
	movwf ZAHL_C5
	movfw TEMP_14
	movwf ZAHL_C4
	return

reconv_datuz				;Unterprogramm zum kombinieren der Digits zu den Werten fr Tag, Monat, Jahr, Stunde und Minute
	movfw ZAHL_B0
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf TAG
	movfw ZAHL_B1
	addwf TAG,f

	movfw ZAHL_B2
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf MONAT
	movfw ZAHL_B3
	addwf MONAT,f

	movfw ZAHL_C0
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf JAHR
	movfw ZAHL_C1
	addwf JAHR,f

	movfw ZAHL_C2
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf STUNDE
	movfw ZAHL_C3
	addwf STUNDE,f

	movfw ZAHL_C4
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf MINUTE
	movfw ZAHL_C5
	addwf MINUTE,f

	movlw d'125'
	movwf TMR2_EXT
	clrf SEKUNDE
	return

convert_regpara				;Unterprogramm zum Zerlegen der Werte fr Kp, Tn und P last in die einzelnen Digits
	call convert_stat_pwm	;kopiere den Status von Stat_PWM Bit 0+7 nach Menu_Stat Bit 2+3, Bit 2 wird im Men verwendet
	movfw P_LAST_L			;kopiere die max. Heizleistung in Zahl_C Register
	movwf ZAHL_C0
	movfw P_LAST_H
	movwf ZAHL_C1
	clrf ZAHL_C2
	clrf ZAHL_C3
	bcf PCLATH,3			;Select Page 0
	call convert_hex_dig	;Unterprogramm zum Zerlegen einer 24-Bit Zahl in die einzelnen Digits (max.999.999)zur Anzeige auf dem Display
	bsf PCLATH,3			;Select Page 1
	movfw DISP_3			;kopiere DISP Register in ZAHL A Register
	movwf ZAHL_A0
	movfw DISP_2
	movwf ZAHL_A1
	movfw DISP_1
	movwf ZAHL_A2
	movfw DISP_0
	movwf ZAHL_A3
	movfw REGLER_KP
	call convert_dig		;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf ZAHL_B1
	movfw TEMP_14
	movwf ZAHL_B0
	movfw REGLER_TN
	call convert_dig		;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf ZAHL_B3
	movfw TEMP_14
	movwf ZAHL_B2
	movlw d'20'
	subwf T_MAX,w			;Umrechnung der Tabellenwerte in Temperatur: (Tabellenwert-20)/2
	movwf ZAHL_C0
	bcf STATUS,C			;Lsche Carry Bit, damit es beim rotieren nicht strt
	rrf ZAHL_C0				;/2
	movfw ZAHL_C0
	call convert_dig		;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf ZAHL_C1
	movfw TEMP_14
	movwf ZAHL_C0
	movfw T_ABSCH			;Abschaltzeit
	call convert_dig		;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf ZAHL_C3
	movfw TEMP_14
	movwf ZAHL_C2
	return

reconv_regpara				;Unterprogramm zum kombinieren der Digits zu den Werten fr Kp, Tn und P last
	call reconv_stat_pwm	;kopiere den Status von Menu_Stat Bit 2+3 nach Stat_PWM Bit 0+7
	call regler_on_off		;Unterprogramm zum Ein-/Ausschalten des PWM Ausgangs 

	movfw ZAHL_C2
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf T_ABSCH			;Abschaltzeit
	movfw ZAHL_C3
	addwf T_ABSCH,f
	movfw ZAHL_C0
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf T_MAX
	movfw ZAHL_C1
	addwf T_MAX,f			;Umrechnung der Temperatur in Tabellenwert: (Temperatur x 2 )+20
	bcf STATUS,C			;Lsche Carry Bit, damit es beim rotieren nicht strt
	rlf T_MAX				; x 2
	movlw d'20'				; +20
	addwf T_MAX
	movfw ZAHL_B0
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf REGLER_KP
	movfw ZAHL_B1
	addwf REGLER_KP,f
	movfw ZAHL_B2
	call mul_10				;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf REGLER_TN
	movfw ZAHL_B3
	addwf REGLER_TN,f
	movfw ZAHL_A3			;kopiere Zahl A register in Disp Register
	movwf DISP_0
	movfw ZAHL_A2
	movwf DISP_1
	movfw ZAHL_A1
	movwf DISP_2
	movfw ZAHL_A0
	movwf DISP_3
	clrf DISP_4				;lschen der Register Disp_4 bis Disp_7
	clrf DISP_5
	clrf DISP_6
	clrf DISP_7
	bcf PCLATH,3			;Select Page 0
	call convert_dig_hex	;Unterprogramm zum Umwandeln der einzelnen Digits (max.999.999) in eine 24-Bit Zahl
	bsf PCLATH,3			;Select Page 1
	movfw ZAHL_C0			;kopiere die Daten in P_LAST Register
	movwf P_LAST_L
	movfw ZAHL_C1
	movwf P_LAST_H
	return


convert_p_kenn				;Unterprogramm zum Zerlegen der Werte fr die 3 Byte der Leistungskennung in die einzelnen Digits
	movlw 0x2C				;Adresse fr byte 1 der Leistungskennung
	movwf FSR0				;schiebe Adresse in FSR0 Register
	movlw 0x02				;setze FSR 0H auf Bank 4/5
	movwf FSR0H				;
	moviw FSR0++
	call trennen_hex		;auftrennen einer 8 bit Zahl in obere (Temp 14) und unter 4Bit (Temp 13)
	movfw TEMP_14
	movwf ZAHL_C0
	movfw TEMP_13
	movwf ZAHL_C1
	moviw FSR0++
	call trennen_hex		;auftrennen einer 8 bit Zahl in obere (Temp 14) und unter 4Bit (Temp 13)
	movfw TEMP_14
	movwf ZAHL_C2
	movfw TEMP_13
	movwf ZAHL_C3
	moviw FSR0++
	call trennen_hex		;auftrennen einer 8 bit Zahl in obere (Temp 14) und unter 4Bit (Temp 13)
	movfw TEMP_14
	movwf ZAHL_C4
	movfw TEMP_13
	movwf ZAHL_C5
	return

reconv_p_kenn				;Unterprogramm zum kombinieren der einzelnen Digits in die Werte fr die 3 Byte der Leistungskennung
	movlw 0x2C				;Adresse fr byte 1 der Leistungskennung
	movwf FSR1				;schiebe Adresse in FSR0 Register
	movlw 0x02
	movwf FSR1H
	clrf P_INOUT_H
	clrf P_INOUT_M
	clrf P_INOUT_L

	swapf ZAHL_C0
	movfw ZAHL_C1
	addwf ZAHL_C0
	movfw ZAHL_C0			;Speichern des 1. Byte im File Register und im internen EEPROM
	movwi FSR1++			;			
	movwf TEMP_13
	movlw 0x09				;EEPROM Adresse
	call writeEE	

	swapf ZAHL_C2
	movfw ZAHL_C3
	addwf ZAHL_C2
	movfw ZAHL_C2			;Speichern des 2. Byte im File Register und im internen EEPROM
	movwi FSR1++			;			
	movwf TEMP_13
	movlw 0x0A				;EEPROM Adresse
	call writeEE	

	swapf ZAHL_C4
	movfw ZAHL_C5
	addwf ZAHL_C4
	movfw ZAHL_C4			;Speichern des 3. Byte im File Register und im internen EEPROM
	movwi FSR1++			;			
	movwf TEMP_13
	movlw 0x0B				;EEPROM Adresse
	call writeEE	
	return


convert_stat_pwm			;kopiere den Status von Stat_PWM Bit 0+7 nach Menu_Stat Bit 2+3, Bit 2 wird im Men verwendet
	bsf MENU_STAT,2
	btfss STAT_PWM,0
	bcf MENU_STAT,2
	bsf MENU_STAT,3
	btfss STAT_PWM,7
	bcf MENU_STAT,3
	return

reconv_stat_pwm				;kopiere den Status von Menu_Stat Bit 2+3 nach Stat_PWM Bit 0+7
	clrf STAT_PWM			;Register lschen
	bsf STAT_PWM,0			;kopiere den Status von Menu_Stat Bit 2+3 nach Stat_PWM Bit 0+7
	btfss MENU_STAT,2
	bcf STAT_PWM,0
	bsf STAT_PWM,7
	btfss MENU_STAT,3
	bcf STAT_PWM,7
	return

mul_10						;Unterprogramm zum Multiplizieren einer Zahl mit 10
	movwf TEMP_13
	movwf TEMP_14
	bcf STATUS,C			;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf TEMP_14
	rlf TEMP_14
	rlf TEMP_14
	movfw TEMP_13
	addwf TEMP_14
	addwf TEMP_14
	movfw TEMP_14
	return


dig_zp						;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt 2
	movlw d'9'				;prfen ob der 9. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfsc STATUS,Z			;prfen ob Ergebniss >0<		
	goto dig_zp9			;wenn ja, berspringe diesen Befehl

	clrf FSR0H				;FSR0H auf Bank 0/1 setzen
	movfw M_HOR_CNT			;Wert = 1...8 , plus 0x28 => Adresse fr DISP 7....DISP 0
	addlw 0x28
	movwf FSR0
	incf INDF0				;erhhe DISP 7....DISP 0
	movlw 0x0A				;hole Wert 10 nach W
	subwf INDF0,w			;subtrahiere Wert 10 von DISP 7....DISP 0
	btfsc STATUS,Z			;prfen, ob >0<, d.h. kein berlauf
	clrf INDF0				;berlauf, Rcksetzen mit Wert 0
	goto dig_zp_end
dig_zp9
	incf TEMP_11			;erhhe den Wert
	movlw d'11'				;prfen, ob Wert bergelaufen (>10)
	subwf TEMP_11,w
	btfss STATUS,Z
	goto dig_zp_end			;
	movlw 0x01
	movwf TEMP_11			;rcksetzen des Wertes mit 1
dig_zp_end
	call disp_set_zp		;Anzeige Menpunkt 1
	movlw 0x0F				;vorbereiten fuer Curser Tabelle im Bereich 0x0F00..0x0FFF
	movwf PCLATH			;
	movfw M_HOR_CNT			
	addlw 0xFF
	goto digit_end


dig_datuz
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll, Tag 10er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_B0			;erhhe den Wert
	movlw d'4'				;prfen, ob Wert bergelaufen (>3)
	subwf ZAHL_B0,w
	btfsc STATUS,Z
	clrf ZAHL_B0			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz2
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll, Tag 1er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_B1			;erhhe den Wert
	movlw d'10'				;prfen, ob Wert bergelaufen (>9)
	subwf ZAHL_B1,w
	btfsc STATUS,Z
	clrf ZAHL_B1			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz3
	movlw d'3'				;prfen ob der 3. Wert verndert werden soll, Monat 10er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz4			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_B2			;erhhe den Wert
	movlw d'2'				;prfen, ob Wert bergelaufen (>1)
	subwf ZAHL_B2,w
	btfsc STATUS,Z
	clrf ZAHL_B2			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz4
	movlw d'4'				;prfen ob der 4. Wert verndert werden soll, Monat 1er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz5			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_B3			;erhhe den Wert
	movlw d'10'				;prfen, ob Wert bergelaufen (>9)
	subwf ZAHL_B3,w
	btfsc STATUS,Z
	clrf ZAHL_B3			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz5
	movlw d'5'				;prfen ob der 5. Wert verndert werden soll, Jahr 10er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz6			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_C0			;erhhe den Wert
	movlw d'10'				;prfen, ob Wert bergelaufen (>9)
	subwf ZAHL_C0,w
	btfsc STATUS,Z
	clrf ZAHL_C0			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz6
	movlw d'6'				;prfen ob der 6. Wert verndert werden soll, Jahr 1er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz7			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_C1			;erhhe den Wert
	movlw d'10'				;prfen, ob Wert bergelaufen (>9)
	subwf ZAHL_C1,w
	btfsc STATUS,Z
	clrf ZAHL_C1			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz7
	movlw d'7'				;prfen ob der 7. Wert verndert werden soll, Stunde 10er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz8			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_C2			;erhhe den Wert
	movlw d'3'				;prfen, ob Wert bergelaufen (>2)
	subwf ZAHL_C2,w
	btfsc STATUS,Z
	clrf ZAHL_C2			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz8
	movlw d'8'				;prfen ob der 8. Wert verndert werden soll, Stunde 1er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz9			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_C3			;erhhe den Wert
	movlw d'10'				;prfen, ob Wert bergelaufen (>9)
	subwf ZAHL_C3,w
	btfsc STATUS,Z
	clrf ZAHL_C3			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz9
	movlw d'9'				;prfen ob der 9. Wert verndert werden soll, Minute 10er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz10		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_C4			;erhhe den Wert
	movlw d'6'				;prfen, ob Wert bergelaufen (>5)
	subwf ZAHL_C4,w
	btfsc STATUS,Z
	clrf ZAHL_C4			;rcksetzen des Wertes mit 0
	goto dig_datuz_end
dig_datuz10
	movlw d'10'				;prfen ob der 10. Wert verndert werden soll, Minute 1er
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_datuz_end			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	incf ZAHL_C5			;erhhe den Wert
	movlw d'10'				;prfen, ob Wert bergelaufen (>9)
	subwf ZAHL_C5,w
	btfsc STATUS,Z
	clrf ZAHL_C5			;rcksetzen des Wertes mit 0
dig_datuz_end
	call disp_set_datuz		;Anzeige Menpunkt 2
	movlw 0x0F				;vorbereiten fuer Curser Tabelle im Bereich 0x0F00..0x0FFF
	movwf PCLATH			;
	movfw M_HOR_CNT			;
	addlw 0x08
	goto digit_end


dig_regpara					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt 6
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf M_HOR_CNT,w		;
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto dig_regpara2_13		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss MENU_STAT,2		;Regler ein/aus
	goto $+3
	bcf MENU_STAT,2
	goto dig_regpara_end
	bsf MENU_STAT,2
	goto dig_regpara_end
dig_regpara2_13
	clrf FSR0H				;FSR0H auf Bank 0/1 setzen
	movfw M_HOR_CNT			;Wert = 2...13 , plus 0x6E => Adresse fr Zahl A0....Zahl C3
	addlw 0x6E
	movwf FSR0
	incf INDF0				;erhhe Zahl A0....Zahl C3
	movlw 0x0A				;hole Wert 10 nach W
	subwf INDF0,w			;subtrahiere Wert 10 von Zahl A0....Zahl C3
	btfsc STATUS,Z			;prfen, ob >0<, d.h. kein berlauf
	clrf INDF0				;berlauf, Rcksetzen mit Wert 0
dig_regpara_end
	call disp_set_regpara	;Anzeige Menpunkt 5
	movlw 0x0F				;vorbereiten fuer Curser Tabelle im Bereich 0x0F00..0x0FFF
	movwf PCLATH			;
	movfw M_HOR_CNT			
	addlw d'18'
	goto digit_end	
	

dig_p_kenn
	movfw M_HOR_CNT
	addlw 0x77
	movwf FSR0
	incf INDF0
	movlw 0x10
	subwf INDF0,w
	btfsc STATUS,Z
	clrf INDF0
dig_p_kenn_end
	call disp_set_p_kenn	;Anzeige Menpunkt 7
	movlw 0x0F				;vorbereiten fuer Curser Tabelle im Bereich 0x0F00..0x0FFF
	movwf PCLATH			;
	movfw M_HOR_CNT			
	addlw d'31'
	goto digit_end		


digit_end
	call cursor_tab
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	call wait_250_p1		;Wartezeit 250ms Page 1
	call wait_250_p1		;Wartezeit 250ms Page 1
	bcf MENU_STAT,0			;lsche die Status bits
	return

regler_on_off				;Unterprogramm zum Ein-/Ausschalten des PWM Ausgangs 
	btfss STAT_PWM,0		;prfen, ob Bit =1 (Regler ausschalten)
	goto regler_on			;Bit ist nicht gesetzt, Regler einschalten
regler_off
	BANKSEL TRISA			;Select BANK 1
	bcf ADCON0,ADON			;ADC ausschalten (Temperaturmessung)
	BANKSEL CCPR1L			;Select BANK 5
	clrf CCP1CON			;(PWM Ausgang Regler)
	BANKSEL T4CON			;Select BANK 8
	bcf T4CON,TMR4ON		;Timer 4 einschalten (PWM)
	BANKSEL PORTA			;Select BANK 0
	return
regler_on
	BANKSEL TRISA			;Select BANK 1
	bsf ADCON0,ADON			;ADC einschalten (Temperaturmessung)
	BANKSEL CCPR1L			;Select BANK 5
	movlw b'00001100'		;CCPx-Modus PWM-Mode: Single, PxA,PxB aktive High (PWM Ausgang Regler)
	movwf CCP1CON
	BANKSEL T4CON			;Select BANK 8
	bsf T4CON,TMR4ON		;Timer 4 einschalten (PWM)
	BANKSEL PORTA			;Select BANK 0
	return

tx_on_off					;Unterprogramm zum Ein-/Ausschalten des TX Ausgangs 
	btfss STAT_PWM,7		;prfen, ob Bit =1 (Tx einschalten)
	goto $+5				;Bit ist nicht gesetzt, TX ausschalten
	BANKSEL ANSELA			;Select BANK 3
	bsf TXSTA,TXEN			;Transmit enabled  (Ausgabe Pin/out)
	BANKSEL PORTA			;Select BANK 0
	return
	BANKSEL ANSELA			;Select BANK 3
	bcf TXSTA,TXEN			;Transmit disabled  (Ausgabe Pin/out)
	BANKSEL PORTA			;Select BANK 0
	return


;*******************************************************************************************
;	Unterprogramme zum Lesen/Schreiben des internen EEPROM
;*******************************************************************************************

read_eeprom_data
	movlw 0x00				;EEPROM Adresse gespeicherter Wert fr EEPROM Speicheradresse Zhlerstnde
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf EEPROM_ADR		;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x01				;EEPROM Adresse gespeicherter Kp Wert fr Regler Heizung
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf REGLER_KP			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x02				;EEPROM Adresse gespeicherter Tn Wert fr Regler Heizung
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf REGLER_TN			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x03				;EEPROM Adresse gespeicherte Max. Leistung Heizung, H-Byte
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf P_LAST_H			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x04				;EEPROM Adresse gespeicherte Max. Leistung Heizung, L-Byte 
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf P_LAST_L			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x05				;EEPROM Adresse gespeicherte Max. Temperatur Heizung
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf T_MAX				;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x07				;EEPROM Adresse gespeichertes Statusregister
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf STAT_PWM			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x08				;EEPROM Adresse gespeicherter Wert fr Abschaltzeit, wenn keine Einspeisung
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf T_ABSCH			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x06				;EEPROM Adresse gespeicherter Wert fr Vorteiler Produktionszhler
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	call mul_10
	movwf ZPROD_IMP			;Kopieren der Daten des W-Registers ins F-Register
	movwf ZPROD_PRE			;Vorteiler fr Produktionszhler
	call regler_on_off		;Unterprogramm zum Ein-/Ausschalten des PWM Ausgangs 
	call tx_on_off			;Unterprogramm zum Ein-/Ausschalten des TX Ausgangs 
	return

store_regpara
	movfw REGLER_KP			;Speichern des Kp Wertes im internen EEPROM
	movwf TEMP_13
	movlw 0x01				;EEPROM Adresse
	call writeEE
	movfw REGLER_TN			;Speichern des Tn Wertes im internen EEPROM
	movwf TEMP_13
	movlw 0x02				;EEPROM Adresse
	call writeEE
	movfw P_LAST_H			;Speichern der Heizungsleistung im internen EEPROM, H-Byte
	movwf TEMP_13
	movlw 0x03				;EEPROM Adresse
	call writeEE
	movfw P_LAST_L			;Speichern der Heizungsleistung im internen EEPROM, L-Byte
	movwf TEMP_13
	movlw 0x04				;EEPROM Adresse
	call writeEE
	movfw T_MAX				;Speichern der max. Heizungstemperatur im internen EEPROM, L-Byte
	movwf TEMP_13
	movlw 0x05				;EEPROM Adresse
	call writeEE
	movfw T_ABSCH			;Speichern der Abschaltzeit im internen EEPROM
	movwf TEMP_13
	movlw 0x08				;EEPROM Adresse
	call writeEE
store_stat_pwm
	movfw STAT_PWM			;Speichern des Status Registers im internen EEPROM
	movwf TEMP_13
	movlw 0x07				;EEPROM Adresse
	call writeEE
	return

	
readEE							;Unterprogramm zum lesen der Zelle deren Adresse in W steht
	BANKSEL EEADRL 				;Select BANK 3
	movwf EEADRL 				;Data Memory, Address to read
	bcf EECON1,CFGS 			;Deselect Config space
	bcf EECON1,EEPGD			;Point to DATA memory
	bsf EECON1,RD 				;EE Read
	movf EEDATL,W 				;W = EEDATL
	BANKSEL PORTA				;Select BANK 0
	return


writeEE							;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	BANKSEL EEADRL 				;Select BANK 3
	movwf EEADRL 				;Data Memory Address to write
	BANKSEL PORTA				;Select BANK 0
	movfw TEMP_13				;
	BANKSEL EEADRL 				;Select BANK 3
	movwf EEDATL 				;Data Memory Value to write
	bcf EECON1,CFGS 			;Deselect Configuration space
	bcf EECON1,EEPGD 			;Point to DATA memory
	bsf EECON1,WREN 			;Enable writes
	bcf INTCON,GIE 				;Disable INTs.
	movlw 55h 					;
	movwf EECON2 				;Write 55h
	movlw 0AAh 					;
	movwf EECON2 				;Write AAh
	bsf EECON1,WR 				;Set WR bit to begin write
	bsf INTCON,GIE 				;Enable Interrupts
	bcf EECON1,WREN 			;Disable writes
	btfsc EECON1,WR 			;Wait for write to complete
	goto $-2 					;Done
	BANKSEL PORTA				;Select BANK 0
	return

;*******************************************************************************************
;	Tabellen
;*******************************************************************************************

	ORG 0x0EF2

monatstage_tab				;Datentabelle fr die Anzahl Tage des Monats
	addwf PCL,f

	retlw 0x1F	;Datenpunkt 0, Januar 31 Tage
	retlw 0x1C	;Datenpunkt 1, Februar 28 Tage
	retlw 0x1F	;Datenpunkt 2, Mrz 31 Tage
	retlw 0x1E	;Datenpunkt 3, April 30 Tage
	retlw 0x1F	;Datenpunkt 4, Mai 31 Tage
	retlw 0x1E	;Datenpunkt 5, Juni 30 Tage
	retlw 0x1F	;Datenpunkt 6, July 31 Tage
	retlw 0x1F	;Datenpunkt 7, August 31 Tage
	retlw 0x1E	;Datenpunkt 8, September 30 Tage
	retlw 0x1F	;Datenpunkt 9, Oktober 31 Tage
	retlw 0x1E	;Datenpunkt 10, November 30 Tage
	retlw 0x1F	;Datenpunkt 11, Dezember 31 Tage


	ORG 0x0F00

cursor_tab				;Tabellen fr Cursorposition in den Menpunkten
	addwf PCL,f
	retlw 0x83			;DP 0: Menpunkt 2: Zhler Prod. 100.000er
	retlw 0x84			;DP 1: Menpunkt 2: Zhler Prod. 10.000er
	retlw 0x85			;DP 2: Menpunkt 2: Zhler Prod. 1.000er
	retlw 0x86			;DP 3: Menpunkt 2: Zhler Prod. 100er
	retlw 0x87			;DP 4: Menpunkt 2: Zhler Prod. 10er
	retlw 0x88			;DP 5: Menpunkt 2: Zhler Prod. 1er
	retlw 0x8A			;DP 6: Menpunkt 2: Zhler Prod. 0.1er
	retlw 0x8B			;DP 7: Menpunkt 2: Zhler Prod. 0.01er
	retlw 0xC4			;DP 8: Menpunkt 2: Zhler Prod. Imp/kWh
	retlw 0x86			;DP 9: Menpunkt 3: Datum/Uhrzeit Tag 10er
	retlw 0x87			;DP 10: Menpunkt 3: Datum/Uhrzeit Tag 1er
	retlw 0x89			;DP 11: Menpunkt 3: Datum/Uhrzeit Monat 10er
	retlw 0x8A			;DP 12: Menpunkt 3: Datum/Uhrzeit Monat 1er
	retlw 0x8E			;DP 13: Menpunkt 3: Datum/Uhrzeit Jahr 10er
	retlw 0x8F			;DP 14: Menpunkt 3: Datum/Uhrzeit Jahr 1er
	retlw 0xC6			;DP 15: Menpunkt 3: Datum/Uhrzeit Stunde 10er
	retlw 0xC7			;DP 16: Menpunkt 3: Datum/Uhrzeit Stunde 1er
	retlw 0xC9			;DP 17: Menpunkt 3: Datum/Uhrzeit Minute 10er
	retlw 0xCA			;DP 18: Menpunkt 3: Datum/Uhrzeit Minute 1er
	retlw 0x80			;DP 19: Menpunkt 5: Regler ein/aus
	retlw 0x87			;DP 20: Menpunkt 5: P last 1000er
	retlw 0x88			;DP 21: Menpunkt 5: P last 100er
	retlw 0x89			;DP 22: Menpunkt 5: P last 10er
	retlw 0x8A			;DP 23: Menpunkt 5: P last 1er
	retlw 0xC5			;DP 24: Menpunkt 5: Kp 0,1er
	retlw 0xC6			;DP 25: Menpunkt 5: Kp 0,01er
	retlw 0xCB			;DP 26: Menpunkt 5: Tn 10er
	retlw 0xCC			;DP 27: Menpunkt 5: Tn 1er
	retlw 0xC3			;DP 28: Menpunkt 5: tmax 10er
	retlw 0xC4			;DP 29: Menpunkt 5: tmax 1er
	retlw 0xCB			;DP 30: Menpunkt 5: tabschalt 10er
	retlw 0xCC			;DP 31: Menpunkt 5: tabschalt 1er
	retlw 0x86			;DP 32: Menpunkt 7: Byte 1, obere 4Bit
	retlw 0x87			;DP 33: Menpunkt 5: Byte 1, untere 4Bit
	retlw 0x89			;DP 34: Menpunkt 5: Byte 2, obere 4Bit
	retlw 0x8A			;DP 35: Menpunkt 5: Byte 2, untere 4Bit
	retlw 0x8C			;DP 36: Menpunkt 5: Byte 3, obere 4Bit
	retlw 0x8D			;DP 37: Menpunkt 5: Byte 3, untere 4Bit


	ORG     0x0F28			; 

tab_temperatur		;Umrechnung der Tabellenwerte in Temperatur: Tabellenwert /2 minus 10
	addwf PCL,f

	retlw 0xFE		;DP 0: AD Wert = 21, T = 117 C
	retlw 0xFA		;DP 1: AD Wert = 22, T = 115 C
	retlw 0xF7		;DP 2: AD Wert = 23, T = 113,5 C
	retlw 0xF3		;DP 3: AD Wert = 24, T = 111,5 C
	retlw 0xF0		;DP 4: AD Wert = 25, T = 110 C
	retlw 0xED		;DP 5: AD Wert = 26, T = 108,5 C
	retlw 0xEA		;DP 6: AD Wert = 27, T = 107 C
	retlw 0xE7		;DP 7: AD Wert = 28, T = 105,5 C
	retlw 0xE4		;DP 8: AD Wert = 29, T = 104 C
	retlw 0xE2		;DP 9: AD Wert = 30, T = 103 C
	retlw 0xDF		;DP 10: AD Wert = 31, T = 101,5 C
	retlw 0xDD		;DP 11: AD Wert = 32, T = 100,5 C
	retlw 0xDA		;DP 12: AD Wert = 33, T = 99 C
	retlw 0xD8		;DP 13: AD Wert = 34, T = 98 C
	retlw 0xD6		;DP 14: AD Wert = 35, T = 97 C
	retlw 0xD3		;DP 15: AD Wert = 36, T = 95,5 C
	retlw 0xD1		;DP 16: AD Wert = 37, T = 94,5 C
	retlw 0xCF		;DP 17: AD Wert = 38, T = 93,5 C
	retlw 0xCD		;DP 18: AD Wert = 39, T = 92,5 C
	retlw 0xCB		;DP 19: AD Wert = 40, T = 91,5 C
	retlw 0xC9		;DP 20: AD Wert = 41, T = 90,5 C
	retlw 0xC7		;DP 21: AD Wert = 42, T = 89,5 C
	retlw 0xC6		;DP 22: AD Wert = 43, T = 89 C
	retlw 0xC4		;DP 23: AD Wert = 44, T = 88 C
	retlw 0xC2		;DP 24: AD Wert = 45, T = 87 C
	retlw 0xC0		;DP 25: AD Wert = 46, T = 86 C
	retlw 0xBF		;DP 26: AD Wert = 47, T = 85,5 C
	retlw 0xBD		;DP 27: AD Wert = 48, T = 84,5 C
	retlw 0xBB		;DP 28: AD Wert = 49, T = 83,5 C
	retlw 0xBA		;DP 29: AD Wert = 50, T = 83 C
	retlw 0xB8		;DP 30: AD Wert = 51, T = 82 C
	retlw 0xB7		;DP 31: AD Wert = 52, T = 81,5 C
	retlw 0xB5		;DP 32: AD Wert = 53, T = 80,5 C
	retlw 0xB4		;DP 33: AD Wert = 54, T = 80 C
	retlw 0xB2		;DP 34: AD Wert = 55, T = 79 C
	retlw 0xB1		;DP 35: AD Wert = 56, T = 78,5 C
	retlw 0xAF		;DP 36: AD Wert = 57, T = 77,5 C
	retlw 0xAE		;DP 37: AD Wert = 58, T = 77 C
	retlw 0xAD		;DP 38: AD Wert = 59, T = 76,5 C
	retlw 0xAB		;DP 39: AD Wert = 60, T = 75,5 C
	retlw 0xAA		;DP 40: AD Wert = 61, T = 75 C
	retlw 0xA9		;DP 41: AD Wert = 62, T = 74,5 C
	retlw 0xA7		;DP 42: AD Wert = 63, T = 73,5 C
	retlw 0xA6		;DP 43: AD Wert = 64, T = 73 C
	retlw 0xA5		;DP 44: AD Wert = 65, T = 72,5 C
	retlw 0xA4		;DP 45: AD Wert = 66, T = 72 C
	retlw 0xA3		;DP 46: AD Wert = 67, T = 71,5 C
	retlw 0xA1		;DP 47: AD Wert = 68, T = 70,5 C
	retlw 0xA0		;DP 48: AD Wert = 69, T = 70 C
	retlw 0x9F		;DP 49: AD Wert = 70, T = 69,5 C
	retlw 0x9E		;DP 50: AD Wert = 71, T = 69 C
	retlw 0x9D		;DP 51: AD Wert = 72, T = 68,5 C
	retlw 0x9C		;DP 52: AD Wert = 73, T = 68 C
	retlw 0x9A		;DP 53: AD Wert = 74, T = 67 C
	retlw 0x99		;DP 54: AD Wert = 75, T = 66,5 C
	retlw 0x98		;DP 55: AD Wert = 76, T = 66 C
	retlw 0x97		;DP 56: AD Wert = 77, T = 65,5 C
	retlw 0x96		;DP 57: AD Wert = 78, T = 65 C
	retlw 0x95		;DP 58: AD Wert = 79, T = 64,5 C
	retlw 0x94		;DP 59: AD Wert = 80, T = 64 C
	retlw 0x93		;DP 60: AD Wert = 81, T = 63,5 C
	retlw 0x92		;DP 61: AD Wert = 82, T = 63 C
	retlw 0x91		;DP 62: AD Wert = 83, T = 62,5 C
	retlw 0x90		;DP 63: AD Wert = 84, T = 62 C
	retlw 0x8F		;DP 64: AD Wert = 85, T = 61,5 C
	retlw 0x8E		;DP 65: AD Wert = 86, T = 61 C
	retlw 0x8D		;DP 66: AD Wert = 87, T = 60,5 C
	retlw 0x8C		;DP 67: AD Wert = 88, T = 60 C
	retlw 0x8B		;DP 68: AD Wert = 89, T = 59,5 C
	retlw 0x8A		;DP 69: AD Wert = 90, T = 59 C
	retlw 0x89		;DP 70: AD Wert = 91, T = 58,5 C
	retlw 0x88		;DP 71: AD Wert = 92, T = 58 C
	retlw 0x87		;DP 72: AD Wert = 93, T = 57,5 C
	retlw 0x86		;DP 73: AD Wert = 94, T = 57 C
	retlw 0x85		;DP 74: AD Wert = 95, T = 56,5 C
	retlw 0x85		;DP 75: AD Wert = 96, T = 56,5 C
	retlw 0x84		;DP 76: AD Wert = 97, T = 56 C
	retlw 0x83		;DP 77: AD Wert = 98, T = 55,5 C
	retlw 0x82		;DP 78: AD Wert = 99, T = 55 C
	retlw 0x81		;DP 79: AD Wert = 100, T = 54,5 C
	retlw 0x80		;DP 80: AD Wert = 101, T = 54 C
	retlw 0x7F		;DP 81: AD Wert = 102, T = 53,5 C
	retlw 0x7E		;DP 82: AD Wert = 103, T = 53 C
	retlw 0x7D		;DP 83: AD Wert = 104, T = 52,5 C
	retlw 0x7D		;DP 84: AD Wert = 105, T = 52,5 C
	retlw 0x7C		;DP 85: AD Wert = 106, T = 52 C
	retlw 0x7B		;DP 86: AD Wert = 107, T = 51,5 C
	retlw 0x7A		;DP 87: AD Wert = 108, T = 51 C
	retlw 0x79		;DP 88: AD Wert = 109, T = 50,5 C
	retlw 0x78		;DP 89: AD Wert = 110, T = 50 C
	retlw 0x77		;DP 90: AD Wert = 111, T = 49,5 C
	retlw 0x77		;DP 91: AD Wert = 112, T = 49,5 C
	retlw 0x76		;DP 92: AD Wert = 113, T = 49 C
	retlw 0x75		;DP 93: AD Wert = 114, T = 48,5 C
	retlw 0x74		;DP 94: AD Wert = 115, T = 48 C
	retlw 0x73		;DP 95: AD Wert = 116, T = 47,5 C
	retlw 0x72		;DP 96: AD Wert = 117, T = 47 C
	retlw 0x72		;DP 97: AD Wert = 118, T = 47 C
	retlw 0x71		;DP 98: AD Wert = 119, T = 46,5 C
	retlw 0x70		;DP 99: AD Wert = 120, T = 46 C
	retlw 0x6F		;DP 100: AD Wert = 121, T = 45,5 C
	retlw 0x6E		;DP 101: AD Wert = 122, T = 45 C
	retlw 0x6E		;DP 102: AD Wert = 123, T = 45 C
	retlw 0x6D		;DP 103: AD Wert = 124, T = 44,5 C
	retlw 0x6C		;DP 104: AD Wert = 125, T = 44 C
	retlw 0x6B		;DP 105: AD Wert = 126, T = 43,5 C
	retlw 0x6A		;DP 106: AD Wert = 127, T = 43 C
	retlw 0x6A		;DP 107: AD Wert = 128, T = 43 C
	retlw 0x69		;DP 108: AD Wert = 129, T = 42,5 C
	retlw 0x68		;DP 109: AD Wert = 130, T = 42 C
	retlw 0x67		;DP 110: AD Wert = 131, T = 41,5 C
	retlw 0x66		;DP 111: AD Wert = 132, T = 41 C
	retlw 0x66		;DP 112: AD Wert = 133, T = 41 C
	retlw 0x65		;DP 113: AD Wert = 134, T = 40,5 C
	retlw 0x64		;DP 114: AD Wert = 135, T = 40 C
	retlw 0x63		;DP 115: AD Wert = 136, T = 39,5 C
	retlw 0x63		;DP 116: AD Wert = 137, T = 39,5 C
	retlw 0x62		;DP 117: AD Wert = 138, T = 39 C
	retlw 0x61		;DP 118: AD Wert = 139, T = 38,5 C
	retlw 0x60		;DP 119: AD Wert = 140, T = 38 C
	retlw 0x5F		;DP 120: AD Wert = 141, T = 37,5 C
	retlw 0x5F		;DP 121: AD Wert = 142, T = 37,5 C
	retlw 0x5E		;DP 122: AD Wert = 143, T = 37 C
	retlw 0x5D		;DP 123: AD Wert = 144, T = 36,5 C
	retlw 0x5C		;DP 124: AD Wert = 145, T = 36 C
	retlw 0x5C		;DP 125: AD Wert = 146, T = 36 C
	retlw 0x5B		;DP 126: AD Wert = 147, T = 35,5 C
	retlw 0x5A		;DP 127: AD Wert = 148, T = 35 C
	retlw 0x59		;DP 128: AD Wert = 149, T = 34,5 C
	retlw 0x58		;DP 129: AD Wert = 150, T = 34 C
	retlw 0x58		;DP 130: AD Wert = 151, T = 34 C
	retlw 0x57		;DP 131: AD Wert = 152, T = 33,5 C
	retlw 0x56		;DP 132: AD Wert = 153, T = 33 C
	retlw 0x55		;DP 133: AD Wert = 154, T = 32,5 C
	retlw 0x55		;DP 134: AD Wert = 155, T = 32,5 C
	retlw 0x54		;DP 135: AD Wert = 156, T = 32 C
	retlw 0x53		;DP 136: AD Wert = 157, T = 31,5 C
	retlw 0x52		;DP 137: AD Wert = 158, T = 31 C
	retlw 0x51		;DP 138: AD Wert = 159, T = 30,5 C
	retlw 0x51		;DP 139: AD Wert = 160, T = 30,5 C
	retlw 0x50		;DP 140: AD Wert = 161, T = 30 C
	retlw 0x4F		;DP 141: AD Wert = 162, T = 29,5 C
	retlw 0x4E		;DP 142: AD Wert = 163, T = 29 C
	retlw 0x4D		;DP 143: AD Wert = 164, T = 28,5 C
	retlw 0x4D		;DP 144: AD Wert = 165, T = 28,5 C
	retlw 0x4C		;DP 145: AD Wert = 166, T = 28 C
	retlw 0x4B		;DP 146: AD Wert = 167, T = 27,5 C
	retlw 0x4A		;DP 147: AD Wert = 168, T = 27 C
	retlw 0x49		;DP 148: AD Wert = 169, T = 26,5 C
	retlw 0x49		;DP 149: AD Wert = 170, T = 26,5 C
	retlw 0x48		;DP 150: AD Wert = 171, T = 26 C
	retlw 0x47		;DP 151: AD Wert = 172, T = 25,5 C
	retlw 0x46		;DP 152: AD Wert = 173, T = 25 C
	retlw 0x45		;DP 153: AD Wert = 174, T = 24,5 C
	retlw 0x45		;DP 154: AD Wert = 175, T = 24,5 C
	retlw 0x44		;DP 155: AD Wert = 176, T = 24 C
	retlw 0x43		;DP 156: AD Wert = 177, T = 23,5 C
	retlw 0x42		;DP 157: AD Wert = 178, T = 23 C
	retlw 0x41		;DP 158: AD Wert = 179, T = 22,5 C
	retlw 0x40		;DP 159: AD Wert = 180, T = 22 C
	retlw 0x40		;DP 160: AD Wert = 181, T = 22 C
	retlw 0x3F		;DP 161: AD Wert = 182, T = 21,5 C
	retlw 0x3E		;DP 162: AD Wert = 183, T = 21 C
	retlw 0x3D		;DP 163: AD Wert = 184, T = 20,5 C
	retlw 0x3C		;DP 164: AD Wert = 185, T = 20 C
	retlw 0x3B		;DP 165: AD Wert = 186, T = 19,5 C
	retlw 0x3A		;DP 166: AD Wert = 187, T = 19 C
	retlw 0x3A		;DP 167: AD Wert = 188, T = 19 C
	retlw 0x39		;DP 168: AD Wert = 189, T = 18,5 C
	retlw 0x38		;DP 169: AD Wert = 190, T = 18 C
	retlw 0x37		;DP 170: AD Wert = 191, T = 17,5 C
	retlw 0x36		;DP 171: AD Wert = 192, T = 17 C
	retlw 0x35		;DP 172: AD Wert = 193, T = 16,5 C
	retlw 0x34		;DP 173: AD Wert = 194, T = 16 C
	retlw 0x33		;DP 174: AD Wert = 195, T = 15,5 C
	retlw 0x32		;DP 175: AD Wert = 196, T = 15 C
	retlw 0x31		;DP 176: AD Wert = 197, T = 14,5 C
	retlw 0x30		;DP 177: AD Wert = 198, T = 14 C
	retlw 0x2F		;DP 178: AD Wert = 199, T = 13,5 C
	retlw 0x2E		;DP 179: AD Wert = 200, T = 13 C
	retlw 0x2D		;DP 180: AD Wert = 201, T = 12,5 C
	retlw 0x2C		;DP 181: AD Wert = 202, T = 12 C
	retlw 0x2B		;DP 182: AD Wert = 203, T = 11,5 C
	retlw 0x2A		;DP 183: AD Wert = 204, T = 11 C
	retlw 0x29		;DP 184: AD Wert = 205, T = 10,5 C
	retlw 0x28		;DP 185: AD Wert = 206, T = 10 C
	retlw 0x27		;DP 186: AD Wert = 207, T = 9,5 C
	retlw 0x26		;DP 187: AD Wert = 208, T = 9 C
	retlw 0x25		;DP 188: AD Wert = 209, T = 8,5 C
	retlw 0x24		;DP 189: AD Wert = 210, T = 8 C
	retlw 0x23		;DP 190: AD Wert = 211, T = 7,5 C
	retlw 0x22		;DP 191: AD Wert = 212, T = 7 C
	retlw 0x21		;DP 192: AD Wert = 213, T = 6,5 C
	retlw 0x1F		;DP 193: AD Wert = 214, T = 5,5 C
	retlw 0x1E		;DP 194: AD Wert = 215, T = 5 C
	retlw 0x1D		;DP 195: AD Wert = 216, T = 4,5 C
	retlw 0x1C		;DP 196: AD Wert = 217, T = 4 C
	retlw 0x1B		;DP 197: AD Wert = 218, T = 3,5 C
	retlw 0x19		;DP 198: AD Wert = 219, T = 2,5 C
	retlw 0x18		;DP 199: AD Wert = 220, T = 2 C
	retlw 0x17		;DP 200: AD Wert = 221, T = 1,5 C
	retlw 0x15		;DP 201: AD Wert = 222, T = 0,5 C
	retlw 0x14		;DP 202: AD Wert = 223, T = 0 C
	retlw 0x13		;DP 203: AD Wert = 224, T = -0,5 C
	retlw 0x11		;DP 204: AD Wert = 225, T = -1,5 C
	retlw 0x10		;DP 205: AD Wert = 226, T = -2 C
	retlw 0x0E		;DP 206: AD Wert = 227, T = -3 C
	retlw 0x0D		;DP 207: AD Wert = 228, T = -3,5 C
	retlw 0x0B		;DP 208: AD Wert = 229, T = -4,5 C
	retlw 0x09		;DP 209: AD Wert = 230, T = -5,5 C
	retlw 0x08		;DP 210: AD Wert = 231, T = -6 C
	retlw 0x06		;DP 211: AD Wert = 232, T = -7 C
	retlw 0x04		;DP 212: AD Wert = 233, T = -8 C
	retlw 0x02		;DP 213: AD Wert = 234, T = -9 C
	retlw 0x00		;DP 214: AD Wert = 235, T = -10 C

	END                	; Ende des Programms

